# Generated by ariadne-codegen
# Source: tools/graphql_codegen/projects/

from __future__ import annotations

from typing import Optional

from pydantic import Field

from wandb._pydantic import GQLBase


class RenameProject(GQLBase):
    rename_project: Optional[RenameProjectRenameProject] = Field(alias="renameProject")


class RenameProjectRenameProject(GQLBase):
    project: Optional[RenameProjectRenameProjectProject]
    inserted: Optional[bool]


class RenameProjectRenameProjectProject(GQLBase):
    name: str


RenameProject.model_rebuild()
RenameProjectRenameProject.model_rebuild()
