# Generated by ariadne-codegen
# Source: tools/graphql_codegen/projects/

from __future__ import annotations

from typing import Optional

from pydantic import Field

from wandb._pydantic import GQLBase

from .fragments import RegistryFragment


class UpsertRegistryProject(GQLBase):
    upsert_model: Optional[UpsertRegistryProjectUpsertModel] = Field(
        alias="upsertModel"
    )


class UpsertRegistryProjectUpsertModel(GQLBase):
    project: Optional[RegistryFragment]
    inserted: Optional[bool]


UpsertRegistryProject.model_rebuild()
UpsertRegistryProjectUpsertModel.model_rebuild()
