# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Literal, Optional

from pydantic import Field

from wandb._pydantic import GQLBase, Typename

from .fragments import FilesFragment


class ArtifactCollectionMembershipFiles(GQLBase):
    project: Optional[ArtifactCollectionMembershipFilesProject]


class ArtifactCollectionMembershipFilesProject(GQLBase):
    artifact_collection: Optional[
        ArtifactCollectionMembershipFilesProjectArtifactCollection
    ] = Field(alias="artifactCollection")


class ArtifactCollectionMembershipFilesProjectArtifactCollection(GQLBase):
    typename__: Typename[
        Literal["ArtifactCollection", "ArtifactPortfolio", "ArtifactSequence"]
    ]
    artifact_membership: Optional[
        ArtifactCollectionMembershipFilesProjectArtifactCollectionArtifactMembership
    ] = Field(alias="artifactMembership")


class ArtifactCollectionMembershipFilesProjectArtifactCollectionArtifactMembership(
    GQLBase
):
    files: Optional[FilesFragment]


ArtifactCollectionMembershipFiles.model_rebuild()
ArtifactCollectionMembershipFilesProject.model_rebuild()
ArtifactCollectionMembershipFilesProjectArtifactCollection.model_rebuild()
ArtifactCollectionMembershipFilesProjectArtifactCollectionArtifactMembership.model_rebuild()
