# Generated by ariadne-codegen

from .create_automation import CreateAutomation
from .create_generic_webhook_integration import (
    CreateGenericWebhookIntegration,
    CreateGenericWebhookIntegrationCreateGenericWebhookIntegration,
    CreateGenericWebhookIntegrationCreateGenericWebhookIntegrationIntegrationGenericWebhookIntegration,
    CreateGenericWebhookIntegrationCreateGenericWebhookIntegrationIntegrationIntegration,
)
from .delete_automation import DeleteAutomation
from .enums import (
    AlertSeverity,
    EventTriggeringConditionType,
    TriggeredActionType,
    TriggerScopeType,
)
from .fragments import (
    ArtifactPortfolioScopeFields,
    ArtifactSequenceScopeFields,
    FilterEventFields,
    GenericWebhookActionFields,
    GenericWebhookActionFieldsIntegrationGenericWebhookIntegration,
    GenericWebhookActionFieldsIntegrationIntegration,
    GenericWebhookIntegrationConnectionFields,
    GenericWebhookIntegrationConnectionFieldsEdges,
    GenericWebhookIntegrationConnectionFieldsEdgesNodeGenericWebhookIntegration,
    GenericWebhookIntegrationConnectionFieldsEdgesNodeIntegration,
    GenericWebhookIntegrationFields,
    IntegrationConnectionFields,
    IntegrationConnectionFieldsEdges,
    IntegrationConnectionFieldsEdgesNodeGenericWebhookIntegration,
    IntegrationConnectionFieldsEdgesNodeIntegration,
    IntegrationConnectionFieldsEdgesNodeSlackIntegration,
    NoOpActionFields,
    NotificationActionFields,
    NotificationActionFieldsIntegrationIntegration,
    NotificationActionFieldsIntegrationSlackIntegration,
    PageInfoFields,
    ProjectConnectionFields,
    ProjectConnectionFieldsEdges,
    ProjectConnectionFieldsEdgesNode,
    ProjectScopeFields,
    QueueJobActionFields,
    QueueJobActionFieldsQueue,
    SlackIntegrationConnectionFields,
    SlackIntegrationConnectionFieldsEdges,
    SlackIntegrationConnectionFieldsEdgesNodeIntegration,
    SlackIntegrationConnectionFieldsEdgesNodeSlackIntegration,
    SlackIntegrationFields,
    TriggerFields,
    TriggerFieldsActionGenericWebhookTriggeredAction,
    TriggerFieldsActionNoOpTriggeredAction,
    TriggerFieldsActionNotificationTriggeredAction,
    TriggerFieldsActionQueueJobTriggeredAction,
    TriggerFieldsEventFilterEventTriggeringCondition,
    TriggerFieldsScopeArtifactPortfolio,
    TriggerFieldsScopeArtifactSequence,
    TriggerFieldsScopeProject,
)
from .generic_webhook_integrations_by_entity import (
    GenericWebhookIntegrationsByEntity,
    GenericWebhookIntegrationsByEntityEntity,
)
from .get_automations import GetAutomations, GetAutomationsSearchScope
from .get_automations_by_entity import (
    GetAutomationsByEntity,
    GetAutomationsByEntitySearchScope,
)
from .input_types import (
    CreateFilterTriggerInput,
    CreateGenericWebhookIntegrationInput,
    GenericWebhookActionInput,
    NoOpTriggeredActionInput,
    NotificationActionInput,
    QueueJobActionInput,
    TriggeredActionConfig,
    UpdateFilterTriggerInput,
)
from .integrations_by_entity import IntegrationsByEntity, IntegrationsByEntityEntity
from .operations import (
    CREATE_AUTOMATION_GQL,
    CREATE_GENERIC_WEBHOOK_INTEGRATION_GQL,
    DELETE_AUTOMATION_GQL,
    GENERIC_WEBHOOK_INTEGRATIONS_BY_ENTITY_GQL,
    GET_AUTOMATIONS_BY_ENTITY_GQL,
    GET_AUTOMATIONS_GQL,
    INTEGRATIONS_BY_ENTITY_GQL,
    SLACK_INTEGRATIONS_BY_ENTITY_GQL,
    UPDATE_AUTOMATION_GQL,
)
from .slack_integrations_by_entity import (
    SlackIntegrationsByEntity,
    SlackIntegrationsByEntityEntity,
)
from .update_automation import UpdateAutomation

__all__ = [
    "CREATE_AUTOMATION_GQL",
    "CREATE_GENERIC_WEBHOOK_INTEGRATION_GQL",
    "DELETE_AUTOMATION_GQL",
    "GENERIC_WEBHOOK_INTEGRATIONS_BY_ENTITY_GQL",
    "GET_AUTOMATIONS_BY_ENTITY_GQL",
    "GET_AUTOMATIONS_GQL",
    "INTEGRATIONS_BY_ENTITY_GQL",
    "SLACK_INTEGRATIONS_BY_ENTITY_GQL",
    "UPDATE_AUTOMATION_GQL",
    "GetAutomations",
    "GetAutomationsSearchScope",
    "GetAutomationsByEntity",
    "GetAutomationsByEntitySearchScope",
    "CreateAutomation",
    "UpdateAutomation",
    "DeleteAutomation",
    "IntegrationsByEntity",
    "IntegrationsByEntityEntity",
    "SlackIntegrationsByEntity",
    "SlackIntegrationsByEntityEntity",
    "GenericWebhookIntegrationsByEntity",
    "GenericWebhookIntegrationsByEntityEntity",
    "CreateGenericWebhookIntegration",
    "CreateGenericWebhookIntegrationCreateGenericWebhookIntegration",
    "CreateGenericWebhookIntegrationCreateGenericWebhookIntegrationIntegrationGenericWebhookIntegration",
    "CreateGenericWebhookIntegrationCreateGenericWebhookIntegrationIntegrationIntegration",
    "CreateFilterTriggerInput",
    "CreateGenericWebhookIntegrationInput",
    "GenericWebhookActionInput",
    "NoOpTriggeredActionInput",
    "NotificationActionInput",
    "QueueJobActionInput",
    "TriggeredActionConfig",
    "UpdateFilterTriggerInput",
    "ArtifactPortfolioScopeFields",
    "ArtifactSequenceScopeFields",
    "FilterEventFields",
    "GenericWebhookActionFields",
    "GenericWebhookActionFieldsIntegrationGenericWebhookIntegration",
    "GenericWebhookActionFieldsIntegrationIntegration",
    "GenericWebhookIntegrationConnectionFields",
    "GenericWebhookIntegrationConnectionFieldsEdges",
    "GenericWebhookIntegrationConnectionFieldsEdgesNodeGenericWebhookIntegration",
    "GenericWebhookIntegrationConnectionFieldsEdgesNodeIntegration",
    "GenericWebhookIntegrationFields",
    "IntegrationConnectionFields",
    "IntegrationConnectionFieldsEdges",
    "IntegrationConnectionFieldsEdgesNodeGenericWebhookIntegration",
    "IntegrationConnectionFieldsEdgesNodeIntegration",
    "IntegrationConnectionFieldsEdgesNodeSlackIntegration",
    "NoOpActionFields",
    "NotificationActionFields",
    "NotificationActionFieldsIntegrationIntegration",
    "NotificationActionFieldsIntegrationSlackIntegration",
    "PageInfoFields",
    "ProjectConnectionFields",
    "ProjectConnectionFieldsEdges",
    "ProjectConnectionFieldsEdgesNode",
    "ProjectScopeFields",
    "QueueJobActionFields",
    "QueueJobActionFieldsQueue",
    "SlackIntegrationConnectionFields",
    "SlackIntegrationConnectionFieldsEdges",
    "SlackIntegrationConnectionFieldsEdgesNodeIntegration",
    "SlackIntegrationConnectionFieldsEdgesNodeSlackIntegration",
    "SlackIntegrationFields",
    "TriggerFields",
    "TriggerFieldsActionGenericWebhookTriggeredAction",
    "TriggerFieldsActionNoOpTriggeredAction",
    "TriggerFieldsActionNotificationTriggeredAction",
    "TriggerFieldsActionQueueJobTriggeredAction",
    "TriggerFieldsEventFilterEventTriggeringCondition",
    "TriggerFieldsScopeArtifactPortfolio",
    "TriggerFieldsScopeArtifactSequence",
    "TriggerFieldsScopeProject",
    "AlertSeverity",
    "EventTriggeringConditionType",
    "TriggerScopeType",
    "TriggeredActionType",
]
