# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Optional

from pydantic import Field

from wandb._pydantic import GQLBase


class AddAliases(GQLBase):
    add_aliases: Optional[AddAliasesAddAliases] = Field(alias="addAliases")


class AddAliasesAddAliases(GQLBase):
    success: bool


AddAliases.model_rebuild()
