# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Literal, Optional

from pydantic import Field

from wandb._pydantic import GQLBase, GQLId, Typename


class MoveArtifactCollection(GQLBase):
    move_artifact_sequence: Optional[MoveArtifactCollectionMoveArtifactSequence] = (
        Field(alias="moveArtifactSequence")
    )


class MoveArtifactCollectionMoveArtifactSequence(GQLBase):
    artifact_collection: Optional[
        MoveArtifactCollectionMoveArtifactSequenceArtifactCollection
    ] = Field(alias="artifactCollection")


class MoveArtifactCollectionMoveArtifactSequenceArtifactCollection(GQLBase):
    typename__: Typename[
        Literal["ArtifactCollection", "ArtifactPortfolio", "ArtifactSequence"]
    ]
    id: GQLId
    name: str
    description: Optional[str]


MoveArtifactCollection.model_rebuild()
MoveArtifactCollectionMoveArtifactSequence.model_rebuild()
