# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Optional

from pydantic import Field

from wandb._pydantic import GQLBase

from .fragments import ArtifactTypesFragment


class ProjectArtifactTypes(GQLBase):
    project: Optional[ProjectArtifactTypesProject]


class ProjectArtifactTypesProject(GQLBase):
    artifact_types: ArtifactTypesFragment = Field(alias="artifactTypes")


ProjectArtifactTypes.model_rebuild()
ProjectArtifactTypesProject.model_rebuild()
