# Generated by ariadne-codegen
# Source: tools/graphql_codegen/automations/

from __future__ import annotations

from typing import Optional

from pydantic import Field

from wandb._pydantic import GQLBase

from .fragments import ProjectConnectionFields


class GetAutomations(GQLBase):
    search_scope: Optional[GetAutomationsSearchScope] = Field(alias="searchScope")


class GetAutomationsSearchScope(GQLBase):
    projects: Optional[ProjectConnectionFields]


GetAutomations.model_rebuild()
GetAutomationsSearchScope.model_rebuild()
