# Generated by ariadne-codegen

from .delete_project import DeleteProject, DeleteProjectDeleteModel
from .fetch_registry import FetchRegistry, FetchRegistryEntity
from .fragments import (
    RegistryFragment,
    RegistryFragmentArtifactTypes,
    RegistryFragmentArtifactTypesEdges,
    RegistryFragmentArtifactTypesEdgesNode,
)
from .input_types import ArtifactTypeInput
from .operations import (
    DELETE_PROJECT_GQL,
    FETCH_REGISTRY_GQL,
    RENAME_PROJECT_GQL,
    UPSERT_REGISTRY_PROJECT_GQL,
)
from .rename_project import (
    RenameProject,
    RenameProjectRenameProject,
    RenameProjectRenameProjectProject,
)
from .upsert_registry_project import (
    UpsertRegistryProject,
    UpsertRegistryProjectUpsertModel,
)

__all__ = [
    "DELETE_PROJECT_GQL",
    "FETCH_REGISTRY_GQL",
    "RENAME_PROJECT_GQL",
    "UPSERT_REGISTRY_PROJECT_GQL",
    "FetchRegistry",
    "FetchRegistryEntity",
    "RenameProject",
    "RenameProjectRenameProject",
    "RenameProjectRenameProjectProject",
    "UpsertRegistryProject",
    "UpsertRegistryProjectUpsertModel",
    "DeleteProject",
    "DeleteProjectDeleteModel",
    "ArtifactTypeInput",
    "RegistryFragment",
    "RegistryFragmentArtifactTypes",
    "RegistryFragmentArtifactTypesEdges",
    "RegistryFragmentArtifactTypesEdgesNode",
]
