# Generated by ariadne-codegen
# Source: core/api/graphql/schemas/schema-latest.graphql

from __future__ import annotations

from typing import Any, Optional

from pydantic import Field

from wandb._pydantic import GQLBase


class ArtifactAliasInput(GQLBase):
    artifact_collection_name: str = Field(alias="artifactCollectionName")
    alias: str


class ArtifactCollectionAliasInput(GQLBase):
    alias: str
    entity_name: str = Field(alias="entityName")
    project_name: str = Field(alias="projectName")
    artifact_collection_name: str = Field(alias="artifactCollectionName")


class TagInput(GQLBase):
    tag_category_name: Optional[str] = Field(alias="tagCategoryName", default=None)
    tag_name: str = Field(alias="tagName")
    attributes: Optional[Any] = None
