# Generated by wandb/proto/wandb_internal_codegen.py.  DO NOT EDIT!

from typing import Literal
DEPRECATED_FEATURES = Literal[
    "keras_callback__data_type",
    "run__mode",
    "run__save_no_args",
    "run__join",
    "plots",
    "run__log_sync",
    "init__config_include_keys",
    "init__config_exclude_keys",
    "keras_callback__save_model",
    "langchain_tracer",
    "artifact__get_path",
    "artifactmanifestentry__name",
    "api__artifact_versions",
    "artifact_collection__change_type",
    "run__define_metric_copy",
    "run_disabled",
    "keras_callback",
    "run__define_metric_best_goal",
    "run__finish_quiet",
    "run__reinit_bool",
    "run__get_url",
    "run__project_name",
    "run__get_project_url",
    "run__get_sweep_url",
]

class Deprecated:
    keras_callback__data_type: DEPRECATED_FEATURES = "keras_callback__data_type"
    run__mode: DEPRECATED_FEATURES = "run__mode"
    run__save_no_args: DEPRECATED_FEATURES = "run__save_no_args"
    run__join: DEPRECATED_FEATURES = "run__join"
    plots: DEPRECATED_FEATURES = "plots"
    run__log_sync: DEPRECATED_FEATURES = "run__log_sync"
    init__config_include_keys: DEPRECATED_FEATURES = "init__config_include_keys"
    init__config_exclude_keys: DEPRECATED_FEATURES = "init__config_exclude_keys"
    keras_callback__save_model: DEPRECATED_FEATURES = "keras_callback__save_model"
    langchain_tracer: DEPRECATED_FEATURES = "langchain_tracer"
    artifact__get_path: DEPRECATED_FEATURES = "artifact__get_path"
    artifactmanifestentry__name: DEPRECATED_FEATURES = "artifactmanifestentry__name"
    api__artifact_versions: DEPRECATED_FEATURES = "api__artifact_versions"
    artifact_collection__change_type: DEPRECATED_FEATURES = "artifact_collection__change_type"
    run__define_metric_copy: DEPRECATED_FEATURES = "run__define_metric_copy"
    run_disabled: DEPRECATED_FEATURES = "run_disabled"
    keras_callback: DEPRECATED_FEATURES = "keras_callback"
    run__define_metric_best_goal: DEPRECATED_FEATURES = "run__define_metric_best_goal"
    run__finish_quiet: DEPRECATED_FEATURES = "run__finish_quiet"
    run__reinit_bool: DEPRECATED_FEATURES = "run__reinit_bool"
    run__get_url: DEPRECATED_FEATURES = "run__get_url"
    run__project_name: DEPRECATED_FEATURES = "run__project_name"
    run__get_project_url: DEPRECATED_FEATURES = "run__get_project_url"
    run__get_sweep_url: DEPRECATED_FEATURES = "run__get_sweep_url"
