# Generated by ariadne-codegen
# Source: core/api/graphql/schemas/schema-latest.graphql

from __future__ import annotations

from typing import Optional

from wandb._pydantic import GQLBase


class ArtifactTypeInput(GQLBase):
    name: str
    description: Optional[str] = None
