// Code generated by wincallback.go using 'go generate'. DO NOT EDIT.

//go:build darwin || freebsd || linux

// External code calls into callbackasm at an offset corresponding
// to the callback index. Callbackasm is a table of MOV and B instructions.
// The MOV instruction loads R12 with the callback index, and the
// B instruction branches to callbackasm1.
// callbackasm1 takes the callback index from R12 and
// indexes into an array that stores information about each callback.
// It then calls the Go implementation for that callback.
#include "textflag.h"

TEXT callbackasm(SB), NOSPLIT|NOFRAME, $0
	MOVD $0, R12
	B    callbackasm1(SB)
	MOVD $1, R12
	B    callbackasm1(SB)
	MOVD $2, R12
	B    callbackasm1(SB)
	MOVD $3, R12
	B    callbackasm1(SB)
	MOVD $4, R12
	B    callbackasm1(SB)
	MOVD $5, R12
	B    callbackasm1(SB)
	MOVD $6, R12
	B    callbackasm1(SB)
	MOVD $7, R12
	B    callbackasm1(SB)
	MOVD $8, R12
	B    callbackasm1(SB)
	MOVD $9, R12
	B    callbackasm1(SB)
	MOVD $10, R12
	B    callbackasm1(SB)
	MOVD $11, R12
	B    callbackasm1(SB)
	MOVD $12, R12
	B    callbackasm1(SB)
	MOVD $13, R12
	B    callbackasm1(SB)
	MOVD $14, R12
	B    callbackasm1(SB)
	MOVD $15, R12
	B    callbackasm1(SB)
	MOVD $16, R12
	B    callbackasm1(SB)
	MOVD $17, R12
	B    callbackasm1(SB)
	MOVD $18, R12
	B    callbackasm1(SB)
	MOVD $19, R12
	B    callbackasm1(SB)
	MOVD $20, R12
	B    callbackasm1(SB)
	MOVD $21, R12
	B    callbackasm1(SB)
	MOVD $22, R12
	B    callbackasm1(SB)
	MOVD $23, R12
	B    callbackasm1(SB)
	MOVD $24, R12
	B    callbackasm1(SB)
	MOVD $25, R12
	B    callbackasm1(SB)
	MOVD $26, R12
	B    callbackasm1(SB)
	MOVD $27, R12
	B    callbackasm1(SB)
	MOVD $28, R12
	B    callbackasm1(SB)
	MOVD $29, R12
	B    callbackasm1(SB)
	MOVD $30, R12
	B    callbackasm1(SB)
	MOVD $31, R12
	B    callbackasm1(SB)
	MOVD $32, R12
	B    callbackasm1(SB)
	MOVD $33, R12
	B    callbackasm1(SB)
	MOVD $34, R12
	B    callbackasm1(SB)
	MOVD $35, R12
	B    callbackasm1(SB)
	MOVD $36, R12
	B    callbackasm1(SB)
	MOVD $37, R12
	B    callbackasm1(SB)
	MOVD $38, R12
	B    callbackasm1(SB)
	MOVD $39, R12
	B    callbackasm1(SB)
	MOVD $40, R12
	B    callbackasm1(SB)
	MOVD $41, R12
	B    callbackasm1(SB)
	MOVD $42, R12
	B    callbackasm1(SB)
	MOVD $43, R12
	B    callbackasm1(SB)
	MOVD $44, R12
	B    callbackasm1(SB)
	MOVD $45, R12
	B    callbackasm1(SB)
	MOVD $46, R12
	B    callbackasm1(SB)
	MOVD $47, R12
	B    callbackasm1(SB)
	MOVD $48, R12
	B    callbackasm1(SB)
	MOVD $49, R12
	B    callbackasm1(SB)
	MOVD $50, R12
	B    callbackasm1(SB)
	MOVD $51, R12
	B    callbackasm1(SB)
	MOVD $52, R12
	B    callbackasm1(SB)
	MOVD $53, R12
	B    callbackasm1(SB)
	MOVD $54, R12
	B    callbackasm1(SB)
	MOVD $55, R12
	B    callbackasm1(SB)
	MOVD $56, R12
	B    callbackasm1(SB)
	MOVD $57, R12
	B    callbackasm1(SB)
	MOVD $58, R12
	B    callbackasm1(SB)
	MOVD $59, R12
	B    callbackasm1(SB)
	MOVD $60, R12
	B    callbackasm1(SB)
	MOVD $61, R12
	B    callbackasm1(SB)
	MOVD $62, R12
	B    callbackasm1(SB)
	MOVD $63, R12
	B    callbackasm1(SB)
	MOVD $64, R12
	B    callbackasm1(SB)
	MOVD $65, R12
	B    callbackasm1(SB)
	MOVD $66, R12
	B    callbackasm1(SB)
	MOVD $67, R12
	B    callbackasm1(SB)
	MOVD $68, R12
	B    callbackasm1(SB)
	MOVD $69, R12
	B    callbackasm1(SB)
	MOVD $70, R12
	B    callbackasm1(SB)
	MOVD $71, R12
	B    callbackasm1(SB)
	MOVD $72, R12
	B    callbackasm1(SB)
	MOVD $73, R12
	B    callbackasm1(SB)
	MOVD $74, R12
	B    callbackasm1(SB)
	MOVD $75, R12
	B    callbackasm1(SB)
	MOVD $76, R12
	B    callbackasm1(SB)
	MOVD $77, R12
	B    callbackasm1(SB)
	MOVD $78, R12
	B    callbackasm1(SB)
	MOVD $79, R12
	B    callbackasm1(SB)
	MOVD $80, R12
	B    callbackasm1(SB)
	MOVD $81, R12
	B    callbackasm1(SB)
	MOVD $82, R12
	B    callbackasm1(SB)
	MOVD $83, R12
	B    callbackasm1(SB)
	MOVD $84, R12
	B    callbackasm1(SB)
	MOVD $85, R12
	B    callbackasm1(SB)
	MOVD $86, R12
	B    callbackasm1(SB)
	MOVD $87, R12
	B    callbackasm1(SB)
	MOVD $88, R12
	B    callbackasm1(SB)
	MOVD $89, R12
	B    callbackasm1(SB)
	MOVD $90, R12
	B    callbackasm1(SB)
	MOVD $91, R12
	B    callbackasm1(SB)
	MOVD $92, R12
	B    callbackasm1(SB)
	MOVD $93, R12
	B    callbackasm1(SB)
	MOVD $94, R12
	B    callbackasm1(SB)
	MOVD $95, R12
	B    callbackasm1(SB)
	MOVD $96, R12
	B    callbackasm1(SB)
	MOVD $97, R12
	B    callbackasm1(SB)
	MOVD $98, R12
	B    callbackasm1(SB)
	MOVD $99, R12
	B    callbackasm1(SB)
	MOVD $100, R12
	B    callbackasm1(SB)
	MOVD $101, R12
	B    callbackasm1(SB)
	MOVD $102, R12
	B    callbackasm1(SB)
	MOVD $103, R12
	B    callbackasm1(SB)
	MOVD $104, R12
	B    callbackasm1(SB)
	MOVD $105, R12
	B    callbackasm1(SB)
	MOVD $106, R12
	B    callbackasm1(SB)
	MOVD $107, R12
	B    callbackasm1(SB)
	MOVD $108, R12
	B    callbackasm1(SB)
	MOVD $109, R12
	B    callbackasm1(SB)
	MOVD $110, R12
	B    callbackasm1(SB)
	MOVD $111, R12
	B    callbackasm1(SB)
	MOVD $112, R12
	B    callbackasm1(SB)
	MOVD $113, R12
	B    callbackasm1(SB)
	MOVD $114, R12
	B    callbackasm1(SB)
	MOVD $115, R12
	B    callbackasm1(SB)
	MOVD $116, R12
	B    callbackasm1(SB)
	MOVD $117, R12
	B    callbackasm1(SB)
	MOVD $118, R12
	B    callbackasm1(SB)
	MOVD $119, R12
	B    callbackasm1(SB)
	MOVD $120, R12
	B    callbackasm1(SB)
	MOVD $121, R12
	B    callbackasm1(SB)
	MOVD $122, R12
	B    callbackasm1(SB)
	MOVD $123, R12
	B    callbackasm1(SB)
	MOVD $124, R12
	B    callbackasm1(SB)
	MOVD $125, R12
	B    callbackasm1(SB)
	MOVD $126, R12
	B    callbackasm1(SB)
	MOVD $127, R12
	B    callbackasm1(SB)
	MOVD $128, R12
	B    callbackasm1(SB)
	MOVD $129, R12
	B    callbackasm1(SB)
	MOVD $130, R12
	B    callbackasm1(SB)
	MOVD $131, R12
	B    callbackasm1(SB)
	MOVD $132, R12
	B    callbackasm1(SB)
	MOVD $133, R12
	B    callbackasm1(SB)
	MOVD $134, R12
	B    callbackasm1(SB)
	MOVD $135, R12
	B    callbackasm1(SB)
	MOVD $136, R12
	B    callbackasm1(SB)
	MOVD $137, R12
	B    callbackasm1(SB)
	MOVD $138, R12
	B    callbackasm1(SB)
	MOVD $139, R12
	B    callbackasm1(SB)
	MOVD $140, R12
	B    callbackasm1(SB)
	MOVD $141, R12
	B    callbackasm1(SB)
	MOVD $142, R12
	B    callbackasm1(SB)
	MOVD $143, R12
	B    callbackasm1(SB)
	MOVD $144, R12
	B    callbackasm1(SB)
	MOVD $145, R12
	B    callbackasm1(SB)
	MOVD $146, R12
	B    callbackasm1(SB)
	MOVD $147, R12
	B    callbackasm1(SB)
	MOVD $148, R12
	B    callbackasm1(SB)
	MOVD $149, R12
	B    callbackasm1(SB)
	MOVD $150, R12
	B    callbackasm1(SB)
	MOVD $151, R12
	B    callbackasm1(SB)
	MOVD $152, R12
	B    callbackasm1(SB)
	MOVD $153, R12
	B    callbackasm1(SB)
	MOVD $154, R12
	B    callbackasm1(SB)
	MOVD $155, R12
	B    callbackasm1(SB)
	MOVD $156, R12
	B    callbackasm1(SB)
	MOVD $157, R12
	B    callbackasm1(SB)
	MOVD $158, R12
	B    callbackasm1(SB)
	MOVD $159, R12
	B    callbackasm1(SB)
	MOVD $160, R12
	B    callbackasm1(SB)
	MOVD $161, R12
	B    callbackasm1(SB)
	MOVD $162, R12
	B    callbackasm1(SB)
	MOVD $163, R12
	B    callbackasm1(SB)
	MOVD $164, R12
	B    callbackasm1(SB)
	MOVD $165, R12
	B    callbackasm1(SB)
	MOVD $166, R12
	B    callbackasm1(SB)
	MOVD $167, R12
	B    callbackasm1(SB)
	MOVD $168, R12
	B    callbackasm1(SB)
	MOVD $169, R12
	B    callbackasm1(SB)
	MOVD $170, R12
	B    callbackasm1(SB)
	MOVD $171, R12
	B    callbackasm1(SB)
	MOVD $172, R12
	B    callbackasm1(SB)
	MOVD $173, R12
	B    callbackasm1(SB)
	MOVD $174, R12
	B    callbackasm1(SB)
	MOVD $175, R12
	B    callbackasm1(SB)
	MOVD $176, R12
	B    callbackasm1(SB)
	MOVD $177, R12
	B    callbackasm1(SB)
	MOVD $178, R12
	B    callbackasm1(SB)
	MOVD $179, R12
	B    callbackasm1(SB)
	MOVD $180, R12
	B    callbackasm1(SB)
	MOVD $181, R12
	B    callbackasm1(SB)
	MOVD $182, R12
	B    callbackasm1(SB)
	MOVD $183, R12
	B    callbackasm1(SB)
	MOVD $184, R12
	B    callbackasm1(SB)
	MOVD $185, R12
	B    callbackasm1(SB)
	MOVD $186, R12
	B    callbackasm1(SB)
	MOVD $187, R12
	B    callbackasm1(SB)
	MOVD $188, R12
	B    callbackasm1(SB)
	MOVD $189, R12
	B    callbackasm1(SB)
	MOVD $190, R12
	B    callbackasm1(SB)
	MOVD $191, R12
	B    callbackasm1(SB)
	MOVD $192, R12
	B    callbackasm1(SB)
	MOVD $193, R12
	B    callbackasm1(SB)
	MOVD $194, R12
	B    callbackasm1(SB)
	MOVD $195, R12
	B    callbackasm1(SB)
	MOVD $196, R12
	B    callbackasm1(SB)
	MOVD $197, R12
	B    callbackasm1(SB)
	MOVD $198, R12
	B    callbackasm1(SB)
	MOVD $199, R12
	B    callbackasm1(SB)
	MOVD $200, R12
	B    callbackasm1(SB)
	MOVD $201, R12
	B    callbackasm1(SB)
	MOVD $202, R12
	B    callbackasm1(SB)
	MOVD $203, R12
	B    callbackasm1(SB)
	MOVD $204, R12
	B    callbackasm1(SB)
	MOVD $205, R12
	B    callbackasm1(SB)
	MOVD $206, R12
	B    callbackasm1(SB)
	MOVD $207, R12
	B    callbackasm1(SB)
	MOVD $208, R12
	B    callbackasm1(SB)
	MOVD $209, R12
	B    callbackasm1(SB)
	MOVD $210, R12
	B    callbackasm1(SB)
	MOVD $211, R12
	B    callbackasm1(SB)
	MOVD $212, R12
	B    callbackasm1(SB)
	MOVD $213, R12
	B    callbackasm1(SB)
	MOVD $214, R12
	B    callbackasm1(SB)
	MOVD $215, R12
	B    callbackasm1(SB)
	MOVD $216, R12
	B    callbackasm1(SB)
	MOVD $217, R12
	B    callbackasm1(SB)
	MOVD $218, R12
	B    callbackasm1(SB)
	MOVD $219, R12
	B    callbackasm1(SB)
	MOVD $220, R12
	B    callbackasm1(SB)
	MOVD $221, R12
	B    callbackasm1(SB)
	MOVD $222, R12
	B    callbackasm1(SB)
	MOVD $223, R12
	B    callbackasm1(SB)
	MOVD $224, R12
	B    callbackasm1(SB)
	MOVD $225, R12
	B    callbackasm1(SB)
	MOVD $226, R12
	B    callbackasm1(SB)
	MOVD $227, R12
	B    callbackasm1(SB)
	MOVD $228, R12
	B    callbackasm1(SB)
	MOVD $229, R12
	B    callbackasm1(SB)
	MOVD $230, R12
	B    callbackasm1(SB)
	MOVD $231, R12
	B    callbackasm1(SB)
	MOVD $232, R12
	B    callbackasm1(SB)
	MOVD $233, R12
	B    callbackasm1(SB)
	MOVD $234, R12
	B    callbackasm1(SB)
	MOVD $235, R12
	B    callbackasm1(SB)
	MOVD $236, R12
	B    callbackasm1(SB)
	MOVD $237, R12
	B    callbackasm1(SB)
	MOVD $238, R12
	B    callbackasm1(SB)
	MOVD $239, R12
	B    callbackasm1(SB)
	MOVD $240, R12
	B    callbackasm1(SB)
	MOVD $241, R12
	B    callbackasm1(SB)
	MOVD $242, R12
	B    callbackasm1(SB)
	MOVD $243, R12
	B    callbackasm1(SB)
	MOVD $244, R12
	B    callbackasm1(SB)
	MOVD $245, R12
	B    callbackasm1(SB)
	MOVD $246, R12
	B    callbackasm1(SB)
	MOVD $247, R12
	B    callbackasm1(SB)
	MOVD $248, R12
	B    callbackasm1(SB)
	MOVD $249, R12
	B    callbackasm1(SB)
	MOVD $250, R12
	B    callbackasm1(SB)
	MOVD $251, R12
	B    callbackasm1(SB)
	MOVD $252, R12
	B    callbackasm1(SB)
	MOVD $253, R12
	B    callbackasm1(SB)
	MOVD $254, R12
	B    callbackasm1(SB)
	MOVD $255, R12
	B    callbackasm1(SB)
	MOVD $256, R12
	B    callbackasm1(SB)
	MOVD $257, R12
	B    callbackasm1(SB)
	MOVD $258, R12
	B    callbackasm1(SB)
	MOVD $259, R12
	B    callbackasm1(SB)
	MOVD $260, R12
	B    callbackasm1(SB)
	MOVD $261, R12
	B    callbackasm1(SB)
	MOVD $262, R12
	B    callbackasm1(SB)
	MOVD $263, R12
	B    callbackasm1(SB)
	MOVD $264, R12
	B    callbackasm1(SB)
	MOVD $265, R12
	B    callbackasm1(SB)
	MOVD $266, R12
	B    callbackasm1(SB)
	MOVD $267, R12
	B    callbackasm1(SB)
	MOVD $268, R12
	B    callbackasm1(SB)
	MOVD $269, R12
	B    callbackasm1(SB)
	MOVD $270, R12
	B    callbackasm1(SB)
	MOVD $271, R12
	B    callbackasm1(SB)
	MOVD $272, R12
	B    callbackasm1(SB)
	MOVD $273, R12
	B    callbackasm1(SB)
	MOVD $274, R12
	B    callbackasm1(SB)
	MOVD $275, R12
	B    callbackasm1(SB)
	MOVD $276, R12
	B    callbackasm1(SB)
	MOVD $277, R12
	B    callbackasm1(SB)
	MOVD $278, R12
	B    callbackasm1(SB)
	MOVD $279, R12
	B    callbackasm1(SB)
	MOVD $280, R12
	B    callbackasm1(SB)
	MOVD $281, R12
	B    callbackasm1(SB)
	MOVD $282, R12
	B    callbackasm1(SB)
	MOVD $283, R12
	B    callbackasm1(SB)
	MOVD $284, R12
	B    callbackasm1(SB)
	MOVD $285, R12
	B    callbackasm1(SB)
	MOVD $286, R12
	B    callbackasm1(SB)
	MOVD $287, R12
	B    callbackasm1(SB)
	MOVD $288, R12
	B    callbackasm1(SB)
	MOVD $289, R12
	B    callbackasm1(SB)
	MOVD $290, R12
	B    callbackasm1(SB)
	MOVD $291, R12
	B    callbackasm1(SB)
	MOVD $292, R12
	B    callbackasm1(SB)
	MOVD $293, R12
	B    callbackasm1(SB)
	MOVD $294, R12
	B    callbackasm1(SB)
	MOVD $295, R12
	B    callbackasm1(SB)
	MOVD $296, R12
	B    callbackasm1(SB)
	MOVD $297, R12
	B    callbackasm1(SB)
	MOVD $298, R12
	B    callbackasm1(SB)
	MOVD $299, R12
	B    callbackasm1(SB)
	MOVD $300, R12
	B    callbackasm1(SB)
	MOVD $301, R12
	B    callbackasm1(SB)
	MOVD $302, R12
	B    callbackasm1(SB)
	MOVD $303, R12
	B    callbackasm1(SB)
	MOVD $304, R12
	B    callbackasm1(SB)
	MOVD $305, R12
	B    callbackasm1(SB)
	MOVD $306, R12
	B    callbackasm1(SB)
	MOVD $307, R12
	B    callbackasm1(SB)
	MOVD $308, R12
	B    callbackasm1(SB)
	MOVD $309, R12
	B    callbackasm1(SB)
	MOVD $310, R12
	B    callbackasm1(SB)
	MOVD $311, R12
	B    callbackasm1(SB)
	MOVD $312, R12
	B    callbackasm1(SB)
	MOVD $313, R12
	B    callbackasm1(SB)
	MOVD $314, R12
	B    callbackasm1(SB)
	MOVD $315, R12
	B    callbackasm1(SB)
	MOVD $316, R12
	B    callbackasm1(SB)
	MOVD $317, R12
	B    callbackasm1(SB)
	MOVD $318, R12
	B    callbackasm1(SB)
	MOVD $319, R12
	B    callbackasm1(SB)
	MOVD $320, R12
	B    callbackasm1(SB)
	MOVD $321, R12
	B    callbackasm1(SB)
	MOVD $322, R12
	B    callbackasm1(SB)
	MOVD $323, R12
	B    callbackasm1(SB)
	MOVD $324, R12
	B    callbackasm1(SB)
	MOVD $325, R12
	B    callbackasm1(SB)
	MOVD $326, R12
	B    callbackasm1(SB)
	MOVD $327, R12
	B    callbackasm1(SB)
	MOVD $328, R12
	B    callbackasm1(SB)
	MOVD $329, R12
	B    callbackasm1(SB)
	MOVD $330, R12
	B    callbackasm1(SB)
	MOVD $331, R12
	B    callbackasm1(SB)
	MOVD $332, R12
	B    callbackasm1(SB)
	MOVD $333, R12
	B    callbackasm1(SB)
	MOVD $334, R12
	B    callbackasm1(SB)
	MOVD $335, R12
	B    callbackasm1(SB)
	MOVD $336, R12
	B    callbackasm1(SB)
	MOVD $337, R12
	B    callbackasm1(SB)
	MOVD $338, R12
	B    callbackasm1(SB)
	MOVD $339, R12
	B    callbackasm1(SB)
	MOVD $340, R12
	B    callbackasm1(SB)
	MOVD $341, R12
	B    callbackasm1(SB)
	MOVD $342, R12
	B    callbackasm1(SB)
	MOVD $343, R12
	B    callbackasm1(SB)
	MOVD $344, R12
	B    callbackasm1(SB)
	MOVD $345, R12
	B    callbackasm1(SB)
	MOVD $346, R12
	B    callbackasm1(SB)
	MOVD $347, R12
	B    callbackasm1(SB)
	MOVD $348, R12
	B    callbackasm1(SB)
	MOVD $349, R12
	B    callbackasm1(SB)
	MOVD $350, R12
	B    callbackasm1(SB)
	MOVD $351, R12
	B    callbackasm1(SB)
	MOVD $352, R12
	B    callbackasm1(SB)
	MOVD $353, R12
	B    callbackasm1(SB)
	MOVD $354, R12
	B    callbackasm1(SB)
	MOVD $355, R12
	B    callbackasm1(SB)
	MOVD $356, R12
	B    callbackasm1(SB)
	MOVD $357, R12
	B    callbackasm1(SB)
	MOVD $358, R12
	B    callbackasm1(SB)
	MOVD $359, R12
	B    callbackasm1(SB)
	MOVD $360, R12
	B    callbackasm1(SB)
	MOVD $361, R12
	B    callbackasm1(SB)
	MOVD $362, R12
	B    callbackasm1(SB)
	MOVD $363, R12
	B    callbackasm1(SB)
	MOVD $364, R12
	B    callbackasm1(SB)
	MOVD $365, R12
	B    callbackasm1(SB)
	MOVD $366, R12
	B    callbackasm1(SB)
	MOVD $367, R12
	B    callbackasm1(SB)
	MOVD $368, R12
	B    callbackasm1(SB)
	MOVD $369, R12
	B    callbackasm1(SB)
	MOVD $370, R12
	B    callbackasm1(SB)
	MOVD $371, R12
	B    callbackasm1(SB)
	MOVD $372, R12
	B    callbackasm1(SB)
	MOVD $373, R12
	B    callbackasm1(SB)
	MOVD $374, R12
	B    callbackasm1(SB)
	MOVD $375, R12
	B    callbackasm1(SB)
	MOVD $376, R12
	B    callbackasm1(SB)
	MOVD $377, R12
	B    callbackasm1(SB)
	MOVD $378, R12
	B    callbackasm1(SB)
	MOVD $379, R12
	B    callbackasm1(SB)
	MOVD $380, R12
	B    callbackasm1(SB)
	MOVD $381, R12
	B    callbackasm1(SB)
	MOVD $382, R12
	B    callbackasm1(SB)
	MOVD $383, R12
	B    callbackasm1(SB)
	MOVD $384, R12
	B    callbackasm1(SB)
	MOVD $385, R12
	B    callbackasm1(SB)
	MOVD $386, R12
	B    callbackasm1(SB)
	MOVD $387, R12
	B    callbackasm1(SB)
	MOVD $388, R12
	B    callbackasm1(SB)
	MOVD $389, R12
	B    callbackasm1(SB)
	MOVD $390, R12
	B    callbackasm1(SB)
	MOVD $391, R12
	B    callbackasm1(SB)
	MOVD $392, R12
	B    callbackasm1(SB)
	MOVD $393, R12
	B    callbackasm1(SB)
	MOVD $394, R12
	B    callbackasm1(SB)
	MOVD $395, R12
	B    callbackasm1(SB)
	MOVD $396, R12
	B    callbackasm1(SB)
	MOVD $397, R12
	B    callbackasm1(SB)
	MOVD $398, R12
	B    callbackasm1(SB)
	MOVD $399, R12
	B    callbackasm1(SB)
	MOVD $400, R12
	B    callbackasm1(SB)
	MOVD $401, R12
	B    callbackasm1(SB)
	MOVD $402, R12
	B    callbackasm1(SB)
	MOVD $403, R12
	B    callbackasm1(SB)
	MOVD $404, R12
	B    callbackasm1(SB)
	MOVD $405, R12
	B    callbackasm1(SB)
	MOVD $406, R12
	B    callbackasm1(SB)
	MOVD $407, R12
	B    callbackasm1(SB)
	MOVD $408, R12
	B    callbackasm1(SB)
	MOVD $409, R12
	B    callbackasm1(SB)
	MOVD $410, R12
	B    callbackasm1(SB)
	MOVD $411, R12
	B    callbackasm1(SB)
	MOVD $412, R12
	B    callbackasm1(SB)
	MOVD $413, R12
	B    callbackasm1(SB)
	MOVD $414, R12
	B    callbackasm1(SB)
	MOVD $415, R12
	B    callbackasm1(SB)
	MOVD $416, R12
	B    callbackasm1(SB)
	MOVD $417, R12
	B    callbackasm1(SB)
	MOVD $418, R12
	B    callbackasm1(SB)
	MOVD $419, R12
	B    callbackasm1(SB)
	MOVD $420, R12
	B    callbackasm1(SB)
	MOVD $421, R12
	B    callbackasm1(SB)
	MOVD $422, R12
	B    callbackasm1(SB)
	MOVD $423, R12
	B    callbackasm1(SB)
	MOVD $424, R12
	B    callbackasm1(SB)
	MOVD $425, R12
	B    callbackasm1(SB)
	MOVD $426, R12
	B    callbackasm1(SB)
	MOVD $427, R12
	B    callbackasm1(SB)
	MOVD $428, R12
	B    callbackasm1(SB)
	MOVD $429, R12
	B    callbackasm1(SB)
	MOVD $430, R12
	B    callbackasm1(SB)
	MOVD $431, R12
	B    callbackasm1(SB)
	MOVD $432, R12
	B    callbackasm1(SB)
	MOVD $433, R12
	B    callbackasm1(SB)
	MOVD $434, R12
	B    callbackasm1(SB)
	MOVD $435, R12
	B    callbackasm1(SB)
	MOVD $436, R12
	B    callbackasm1(SB)
	MOVD $437, R12
	B    callbackasm1(SB)
	MOVD $438, R12
	B    callbackasm1(SB)
	MOVD $439, R12
	B    callbackasm1(SB)
	MOVD $440, R12
	B    callbackasm1(SB)
	MOVD $441, R12
	B    callbackasm1(SB)
	MOVD $442, R12
	B    callbackasm1(SB)
	MOVD $443, R12
	B    callbackasm1(SB)
	MOVD $444, R12
	B    callbackasm1(SB)
	MOVD $445, R12
	B    callbackasm1(SB)
	MOVD $446, R12
	B    callbackasm1(SB)
	MOVD $447, R12
	B    callbackasm1(SB)
	MOVD $448, R12
	B    callbackasm1(SB)
	MOVD $449, R12
	B    callbackasm1(SB)
	MOVD $450, R12
	B    callbackasm1(SB)
	MOVD $451, R12
	B    callbackasm1(SB)
	MOVD $452, R12
	B    callbackasm1(SB)
	MOVD $453, R12
	B    callbackasm1(SB)
	MOVD $454, R12
	B    callbackasm1(SB)
	MOVD $455, R12
	B    callbackasm1(SB)
	MOVD $456, R12
	B    callbackasm1(SB)
	MOVD $457, R12
	B    callbackasm1(SB)
	MOVD $458, R12
	B    callbackasm1(SB)
	MOVD $459, R12
	B    callbackasm1(SB)
	MOVD $460, R12
	B    callbackasm1(SB)
	MOVD $461, R12
	B    callbackasm1(SB)
	MOVD $462, R12
	B    callbackasm1(SB)
	MOVD $463, R12
	B    callbackasm1(SB)
	MOVD $464, R12
	B    callbackasm1(SB)
	MOVD $465, R12
	B    callbackasm1(SB)
	MOVD $466, R12
	B    callbackasm1(SB)
	MOVD $467, R12
	B    callbackasm1(SB)
	MOVD $468, R12
	B    callbackasm1(SB)
	MOVD $469, R12
	B    callbackasm1(SB)
	MOVD $470, R12
	B    callbackasm1(SB)
	MOVD $471, R12
	B    callbackasm1(SB)
	MOVD $472, R12
	B    callbackasm1(SB)
	MOVD $473, R12
	B    callbackasm1(SB)
	MOVD $474, R12
	B    callbackasm1(SB)
	MOVD $475, R12
	B    callbackasm1(SB)
	MOVD $476, R12
	B    callbackasm1(SB)
	MOVD $477, R12
	B    callbackasm1(SB)
	MOVD $478, R12
	B    callbackasm1(SB)
	MOVD $479, R12
	B    callbackasm1(SB)
	MOVD $480, R12
	B    callbackasm1(SB)
	MOVD $481, R12
	B    callbackasm1(SB)
	MOVD $482, R12
	B    callbackasm1(SB)
	MOVD $483, R12
	B    callbackasm1(SB)
	MOVD $484, R12
	B    callbackasm1(SB)
	MOVD $485, R12
	B    callbackasm1(SB)
	MOVD $486, R12
	B    callbackasm1(SB)
	MOVD $487, R12
	B    callbackasm1(SB)
	MOVD $488, R12
	B    callbackasm1(SB)
	MOVD $489, R12
	B    callbackasm1(SB)
	MOVD $490, R12
	B    callbackasm1(SB)
	MOVD $491, R12
	B    callbackasm1(SB)
	MOVD $492, R12
	B    callbackasm1(SB)
	MOVD $493, R12
	B    callbackasm1(SB)
	MOVD $494, R12
	B    callbackasm1(SB)
	MOVD $495, R12
	B    callbackasm1(SB)
	MOVD $496, R12
	B    callbackasm1(SB)
	MOVD $497, R12
	B    callbackasm1(SB)
	MOVD $498, R12
	B    callbackasm1(SB)
	MOVD $499, R12
	B    callbackasm1(SB)
	MOVD $500, R12
	B    callbackasm1(SB)
	MOVD $501, R12
	B    callbackasm1(SB)
	MOVD $502, R12
	B    callbackasm1(SB)
	MOVD $503, R12
	B    callbackasm1(SB)
	MOVD $504, R12
	B    callbackasm1(SB)
	MOVD $505, R12
	B    callbackasm1(SB)
	MOVD $506, R12
	B    callbackasm1(SB)
	MOVD $507, R12
	B    callbackasm1(SB)
	MOVD $508, R12
	B    callbackasm1(SB)
	MOVD $509, R12
	B    callbackasm1(SB)
	MOVD $510, R12
	B    callbackasm1(SB)
	MOVD $511, R12
	B    callbackasm1(SB)
	MOVD $512, R12
	B    callbackasm1(SB)
	MOVD $513, R12
	B    callbackasm1(SB)
	MOVD $514, R12
	B    callbackasm1(SB)
	MOVD $515, R12
	B    callbackasm1(SB)
	MOVD $516, R12
	B    callbackasm1(SB)
	MOVD $517, R12
	B    callbackasm1(SB)
	MOVD $518, R12
	B    callbackasm1(SB)
	MOVD $519, R12
	B    callbackasm1(SB)
	MOVD $520, R12
	B    callbackasm1(SB)
	MOVD $521, R12
	B    callbackasm1(SB)
	MOVD $522, R12
	B    callbackasm1(SB)
	MOVD $523, R12
	B    callbackasm1(SB)
	MOVD $524, R12
	B    callbackasm1(SB)
	MOVD $525, R12
	B    callbackasm1(SB)
	MOVD $526, R12
	B    callbackasm1(SB)
	MOVD $527, R12
	B    callbackasm1(SB)
	MOVD $528, R12
	B    callbackasm1(SB)
	MOVD $529, R12
	B    callbackasm1(SB)
	MOVD $530, R12
	B    callbackasm1(SB)
	MOVD $531, R12
	B    callbackasm1(SB)
	MOVD $532, R12
	B    callbackasm1(SB)
	MOVD $533, R12
	B    callbackasm1(SB)
	MOVD $534, R12
	B    callbackasm1(SB)
	MOVD $535, R12
	B    callbackasm1(SB)
	MOVD $536, R12
	B    callbackasm1(SB)
	MOVD $537, R12
	B    callbackasm1(SB)
	MOVD $538, R12
	B    callbackasm1(SB)
	MOVD $539, R12
	B    callbackasm1(SB)
	MOVD $540, R12
	B    callbackasm1(SB)
	MOVD $541, R12
	B    callbackasm1(SB)
	MOVD $542, R12
	B    callbackasm1(SB)
	MOVD $543, R12
	B    callbackasm1(SB)
	MOVD $544, R12
	B    callbackasm1(SB)
	MOVD $545, R12
	B    callbackasm1(SB)
	MOVD $546, R12
	B    callbackasm1(SB)
	MOVD $547, R12
	B    callbackasm1(SB)
	MOVD $548, R12
	B    callbackasm1(SB)
	MOVD $549, R12
	B    callbackasm1(SB)
	MOVD $550, R12
	B    callbackasm1(SB)
	MOVD $551, R12
	B    callbackasm1(SB)
	MOVD $552, R12
	B    callbackasm1(SB)
	MOVD $553, R12
	B    callbackasm1(SB)
	MOVD $554, R12
	B    callbackasm1(SB)
	MOVD $555, R12
	B    callbackasm1(SB)
	MOVD $556, R12
	B    callbackasm1(SB)
	MOVD $557, R12
	B    callbackasm1(SB)
	MOVD $558, R12
	B    callbackasm1(SB)
	MOVD $559, R12
	B    callbackasm1(SB)
	MOVD $560, R12
	B    callbackasm1(SB)
	MOVD $561, R12
	B    callbackasm1(SB)
	MOVD $562, R12
	B    callbackasm1(SB)
	MOVD $563, R12
	B    callbackasm1(SB)
	MOVD $564, R12
	B    callbackasm1(SB)
	MOVD $565, R12
	B    callbackasm1(SB)
	MOVD $566, R12
	B    callbackasm1(SB)
	MOVD $567, R12
	B    callbackasm1(SB)
	MOVD $568, R12
	B    callbackasm1(SB)
	MOVD $569, R12
	B    callbackasm1(SB)
	MOVD $570, R12
	B    callbackasm1(SB)
	MOVD $571, R12
	B    callbackasm1(SB)
	MOVD $572, R12
	B    callbackasm1(SB)
	MOVD $573, R12
	B    callbackasm1(SB)
	MOVD $574, R12
	B    callbackasm1(SB)
	MOVD $575, R12
	B    callbackasm1(SB)
	MOVD $576, R12
	B    callbackasm1(SB)
	MOVD $577, R12
	B    callbackasm1(SB)
	MOVD $578, R12
	B    callbackasm1(SB)
	MOVD $579, R12
	B    callbackasm1(SB)
	MOVD $580, R12
	B    callbackasm1(SB)
	MOVD $581, R12
	B    callbackasm1(SB)
	MOVD $582, R12
	B    callbackasm1(SB)
	MOVD $583, R12
	B    callbackasm1(SB)
	MOVD $584, R12
	B    callbackasm1(SB)
	MOVD $585, R12
	B    callbackasm1(SB)
	MOVD $586, R12
	B    callbackasm1(SB)
	MOVD $587, R12
	B    callbackasm1(SB)
	MOVD $588, R12
	B    callbackasm1(SB)
	MOVD $589, R12
	B    callbackasm1(SB)
	MOVD $590, R12
	B    callbackasm1(SB)
	MOVD $591, R12
	B    callbackasm1(SB)
	MOVD $592, R12
	B    callbackasm1(SB)
	MOVD $593, R12
	B    callbackasm1(SB)
	MOVD $594, R12
	B    callbackasm1(SB)
	MOVD $595, R12
	B    callbackasm1(SB)
	MOVD $596, R12
	B    callbackasm1(SB)
	MOVD $597, R12
	B    callbackasm1(SB)
	MOVD $598, R12
	B    callbackasm1(SB)
	MOVD $599, R12
	B    callbackasm1(SB)
	MOVD $600, R12
	B    callbackasm1(SB)
	MOVD $601, R12
	B    callbackasm1(SB)
	MOVD $602, R12
	B    callbackasm1(SB)
	MOVD $603, R12
	B    callbackasm1(SB)
	MOVD $604, R12
	B    callbackasm1(SB)
	MOVD $605, R12
	B    callbackasm1(SB)
	MOVD $606, R12
	B    callbackasm1(SB)
	MOVD $607, R12
	B    callbackasm1(SB)
	MOVD $608, R12
	B    callbackasm1(SB)
	MOVD $609, R12
	B    callbackasm1(SB)
	MOVD $610, R12
	B    callbackasm1(SB)
	MOVD $611, R12
	B    callbackasm1(SB)
	MOVD $612, R12
	B    callbackasm1(SB)
	MOVD $613, R12
	B    callbackasm1(SB)
	MOVD $614, R12
	B    callbackasm1(SB)
	MOVD $615, R12
	B    callbackasm1(SB)
	MOVD $616, R12
	B    callbackasm1(SB)
	MOVD $617, R12
	B    callbackasm1(SB)
	MOVD $618, R12
	B    callbackasm1(SB)
	MOVD $619, R12
	B    callbackasm1(SB)
	MOVD $620, R12
	B    callbackasm1(SB)
	MOVD $621, R12
	B    callbackasm1(SB)
	MOVD $622, R12
	B    callbackasm1(SB)
	MOVD $623, R12
	B    callbackasm1(SB)
	MOVD $624, R12
	B    callbackasm1(SB)
	MOVD $625, R12
	B    callbackasm1(SB)
	MOVD $626, R12
	B    callbackasm1(SB)
	MOVD $627, R12
	B    callbackasm1(SB)
	MOVD $628, R12
	B    callbackasm1(SB)
	MOVD $629, R12
	B    callbackasm1(SB)
	MOVD $630, R12
	B    callbackasm1(SB)
	MOVD $631, R12
	B    callbackasm1(SB)
	MOVD $632, R12
	B    callbackasm1(SB)
	MOVD $633, R12
	B    callbackasm1(SB)
	MOVD $634, R12
	B    callbackasm1(SB)
	MOVD $635, R12
	B    callbackasm1(SB)
	MOVD $636, R12
	B    callbackasm1(SB)
	MOVD $637, R12
	B    callbackasm1(SB)
	MOVD $638, R12
	B    callbackasm1(SB)
	MOVD $639, R12
	B    callbackasm1(SB)
	MOVD $640, R12
	B    callbackasm1(SB)
	MOVD $641, R12
	B    callbackasm1(SB)
	MOVD $642, R12
	B    callbackasm1(SB)
	MOVD $643, R12
	B    callbackasm1(SB)
	MOVD $644, R12
	B    callbackasm1(SB)
	MOVD $645, R12
	B    callbackasm1(SB)
	MOVD $646, R12
	B    callbackasm1(SB)
	MOVD $647, R12
	B    callbackasm1(SB)
	MOVD $648, R12
	B    callbackasm1(SB)
	MOVD $649, R12
	B    callbackasm1(SB)
	MOVD $650, R12
	B    callbackasm1(SB)
	MOVD $651, R12
	B    callbackasm1(SB)
	MOVD $652, R12
	B    callbackasm1(SB)
	MOVD $653, R12
	B    callbackasm1(SB)
	MOVD $654, R12
	B    callbackasm1(SB)
	MOVD $655, R12
	B    callbackasm1(SB)
	MOVD $656, R12
	B    callbackasm1(SB)
	MOVD $657, R12
	B    callbackasm1(SB)
	MOVD $658, R12
	B    callbackasm1(SB)
	MOVD $659, R12
	B    callbackasm1(SB)
	MOVD $660, R12
	B    callbackasm1(SB)
	MOVD $661, R12
	B    callbackasm1(SB)
	MOVD $662, R12
	B    callbackasm1(SB)
	MOVD $663, R12
	B    callbackasm1(SB)
	MOVD $664, R12
	B    callbackasm1(SB)
	MOVD $665, R12
	B    callbackasm1(SB)
	MOVD $666, R12
	B    callbackasm1(SB)
	MOVD $667, R12
	B    callbackasm1(SB)
	MOVD $668, R12
	B    callbackasm1(SB)
	MOVD $669, R12
	B    callbackasm1(SB)
	MOVD $670, R12
	B    callbackasm1(SB)
	MOVD $671, R12
	B    callbackasm1(SB)
	MOVD $672, R12
	B    callbackasm1(SB)
	MOVD $673, R12
	B    callbackasm1(SB)
	MOVD $674, R12
	B    callbackasm1(SB)
	MOVD $675, R12
	B    callbackasm1(SB)
	MOVD $676, R12
	B    callbackasm1(SB)
	MOVD $677, R12
	B    callbackasm1(SB)
	MOVD $678, R12
	B    callbackasm1(SB)
	MOVD $679, R12
	B    callbackasm1(SB)
	MOVD $680, R12
	B    callbackasm1(SB)
	MOVD $681, R12
	B    callbackasm1(SB)
	MOVD $682, R12
	B    callbackasm1(SB)
	MOVD $683, R12
	B    callbackasm1(SB)
	MOVD $684, R12
	B    callbackasm1(SB)
	MOVD $685, R12
	B    callbackasm1(SB)
	MOVD $686, R12
	B    callbackasm1(SB)
	MOVD $687, R12
	B    callbackasm1(SB)
	MOVD $688, R12
	B    callbackasm1(SB)
	MOVD $689, R12
	B    callbackasm1(SB)
	MOVD $690, R12
	B    callbackasm1(SB)
	MOVD $691, R12
	B    callbackasm1(SB)
	MOVD $692, R12
	B    callbackasm1(SB)
	MOVD $693, R12
	B    callbackasm1(SB)
	MOVD $694, R12
	B    callbackasm1(SB)
	MOVD $695, R12
	B    callbackasm1(SB)
	MOVD $696, R12
	B    callbackasm1(SB)
	MOVD $697, R12
	B    callbackasm1(SB)
	MOVD $698, R12
	B    callbackasm1(SB)
	MOVD $699, R12
	B    callbackasm1(SB)
	MOVD $700, R12
	B    callbackasm1(SB)
	MOVD $701, R12
	B    callbackasm1(SB)
	MOVD $702, R12
	B    callbackasm1(SB)
	MOVD $703, R12
	B    callbackasm1(SB)
	MOVD $704, R12
	B    callbackasm1(SB)
	MOVD $705, R12
	B    callbackasm1(SB)
	MOVD $706, R12
	B    callbackasm1(SB)
	MOVD $707, R12
	B    callbackasm1(SB)
	MOVD $708, R12
	B    callbackasm1(SB)
	MOVD $709, R12
	B    callbackasm1(SB)
	MOVD $710, R12
	B    callbackasm1(SB)
	MOVD $711, R12
	B    callbackasm1(SB)
	MOVD $712, R12
	B    callbackasm1(SB)
	MOVD $713, R12
	B    callbackasm1(SB)
	MOVD $714, R12
	B    callbackasm1(SB)
	MOVD $715, R12
	B    callbackasm1(SB)
	MOVD $716, R12
	B    callbackasm1(SB)
	MOVD $717, R12
	B    callbackasm1(SB)
	MOVD $718, R12
	B    callbackasm1(SB)
	MOVD $719, R12
	B    callbackasm1(SB)
	MOVD $720, R12
	B    callbackasm1(SB)
	MOVD $721, R12
	B    callbackasm1(SB)
	MOVD $722, R12
	B    callbackasm1(SB)
	MOVD $723, R12
	B    callbackasm1(SB)
	MOVD $724, R12
	B    callbackasm1(SB)
	MOVD $725, R12
	B    callbackasm1(SB)
	MOVD $726, R12
	B    callbackasm1(SB)
	MOVD $727, R12
	B    callbackasm1(SB)
	MOVD $728, R12
	B    callbackasm1(SB)
	MOVD $729, R12
	B    callbackasm1(SB)
	MOVD $730, R12
	B    callbackasm1(SB)
	MOVD $731, R12
	B    callbackasm1(SB)
	MOVD $732, R12
	B    callbackasm1(SB)
	MOVD $733, R12
	B    callbackasm1(SB)
	MOVD $734, R12
	B    callbackasm1(SB)
	MOVD $735, R12
	B    callbackasm1(SB)
	MOVD $736, R12
	B    callbackasm1(SB)
	MOVD $737, R12
	B    callbackasm1(SB)
	MOVD $738, R12
	B    callbackasm1(SB)
	MOVD $739, R12
	B    callbackasm1(SB)
	MOVD $740, R12
	B    callbackasm1(SB)
	MOVD $741, R12
	B    callbackasm1(SB)
	MOVD $742, R12
	B    callbackasm1(SB)
	MOVD $743, R12
	B    callbackasm1(SB)
	MOVD $744, R12
	B    callbackasm1(SB)
	MOVD $745, R12
	B    callbackasm1(SB)
	MOVD $746, R12
	B    callbackasm1(SB)
	MOVD $747, R12
	B    callbackasm1(SB)
	MOVD $748, R12
	B    callbackasm1(SB)
	MOVD $749, R12
	B    callbackasm1(SB)
	MOVD $750, R12
	B    callbackasm1(SB)
	MOVD $751, R12
	B    callbackasm1(SB)
	MOVD $752, R12
	B    callbackasm1(SB)
	MOVD $753, R12
	B    callbackasm1(SB)
	MOVD $754, R12
	B    callbackasm1(SB)
	MOVD $755, R12
	B    callbackasm1(SB)
	MOVD $756, R12
	B    callbackasm1(SB)
	MOVD $757, R12
	B    callbackasm1(SB)
	MOVD $758, R12
	B    callbackasm1(SB)
	MOVD $759, R12
	B    callbackasm1(SB)
	MOVD $760, R12
	B    callbackasm1(SB)
	MOVD $761, R12
	B    callbackasm1(SB)
	MOVD $762, R12
	B    callbackasm1(SB)
	MOVD $763, R12
	B    callbackasm1(SB)
	MOVD $764, R12
	B    callbackasm1(SB)
	MOVD $765, R12
	B    callbackasm1(SB)
	MOVD $766, R12
	B    callbackasm1(SB)
	MOVD $767, R12
	B    callbackasm1(SB)
	MOVD $768, R12
	B    callbackasm1(SB)
	MOVD $769, R12
	B    callbackasm1(SB)
	MOVD $770, R12
	B    callbackasm1(SB)
	MOVD $771, R12
	B    callbackasm1(SB)
	MOVD $772, R12
	B    callbackasm1(SB)
	MOVD $773, R12
	B    callbackasm1(SB)
	MOVD $774, R12
	B    callbackasm1(SB)
	MOVD $775, R12
	B    callbackasm1(SB)
	MOVD $776, R12
	B    callbackasm1(SB)
	MOVD $777, R12
	B    callbackasm1(SB)
	MOVD $778, R12
	B    callbackasm1(SB)
	MOVD $779, R12
	B    callbackasm1(SB)
	MOVD $780, R12
	B    callbackasm1(SB)
	MOVD $781, R12
	B    callbackasm1(SB)
	MOVD $782, R12
	B    callbackasm1(SB)
	MOVD $783, R12
	B    callbackasm1(SB)
	MOVD $784, R12
	B    callbackasm1(SB)
	MOVD $785, R12
	B    callbackasm1(SB)
	MOVD $786, R12
	B    callbackasm1(SB)
	MOVD $787, R12
	B    callbackasm1(SB)
	MOVD $788, R12
	B    callbackasm1(SB)
	MOVD $789, R12
	B    callbackasm1(SB)
	MOVD $790, R12
	B    callbackasm1(SB)
	MOVD $791, R12
	B    callbackasm1(SB)
	MOVD $792, R12
	B    callbackasm1(SB)
	MOVD $793, R12
	B    callbackasm1(SB)
	MOVD $794, R12
	B    callbackasm1(SB)
	MOVD $795, R12
	B    callbackasm1(SB)
	MOVD $796, R12
	B    callbackasm1(SB)
	MOVD $797, R12
	B    callbackasm1(SB)
	MOVD $798, R12
	B    callbackasm1(SB)
	MOVD $799, R12
	B    callbackasm1(SB)
	MOVD $800, R12
	B    callbackasm1(SB)
	MOVD $801, R12
	B    callbackasm1(SB)
	MOVD $802, R12
	B    callbackasm1(SB)
	MOVD $803, R12
	B    callbackasm1(SB)
	MOVD $804, R12
	B    callbackasm1(SB)
	MOVD $805, R12
	B    callbackasm1(SB)
	MOVD $806, R12
	B    callbackasm1(SB)
	MOVD $807, R12
	B    callbackasm1(SB)
	MOVD $808, R12
	B    callbackasm1(SB)
	MOVD $809, R12
	B    callbackasm1(SB)
	MOVD $810, R12
	B    callbackasm1(SB)
	MOVD $811, R12
	B    callbackasm1(SB)
	MOVD $812, R12
	B    callbackasm1(SB)
	MOVD $813, R12
	B    callbackasm1(SB)
	MOVD $814, R12
	B    callbackasm1(SB)
	MOVD $815, R12
	B    callbackasm1(SB)
	MOVD $816, R12
	B    callbackasm1(SB)
	MOVD $817, R12
	B    callbackasm1(SB)
	MOVD $818, R12
	B    callbackasm1(SB)
	MOVD $819, R12
	B    callbackasm1(SB)
	MOVD $820, R12
	B    callbackasm1(SB)
	MOVD $821, R12
	B    callbackasm1(SB)
	MOVD $822, R12
	B    callbackasm1(SB)
	MOVD $823, R12
	B    callbackasm1(SB)
	MOVD $824, R12
	B    callbackasm1(SB)
	MOVD $825, R12
	B    callbackasm1(SB)
	MOVD $826, R12
	B    callbackasm1(SB)
	MOVD $827, R12
	B    callbackasm1(SB)
	MOVD $828, R12
	B    callbackasm1(SB)
	MOVD $829, R12
	B    callbackasm1(SB)
	MOVD $830, R12
	B    callbackasm1(SB)
	MOVD $831, R12
	B    callbackasm1(SB)
	MOVD $832, R12
	B    callbackasm1(SB)
	MOVD $833, R12
	B    callbackasm1(SB)
	MOVD $834, R12
	B    callbackasm1(SB)
	MOVD $835, R12
	B    callbackasm1(SB)
	MOVD $836, R12
	B    callbackasm1(SB)
	MOVD $837, R12
	B    callbackasm1(SB)
	MOVD $838, R12
	B    callbackasm1(SB)
	MOVD $839, R12
	B    callbackasm1(SB)
	MOVD $840, R12
	B    callbackasm1(SB)
	MOVD $841, R12
	B    callbackasm1(SB)
	MOVD $842, R12
	B    callbackasm1(SB)
	MOVD $843, R12
	B    callbackasm1(SB)
	MOVD $844, R12
	B    callbackasm1(SB)
	MOVD $845, R12
	B    callbackasm1(SB)
	MOVD $846, R12
	B    callbackasm1(SB)
	MOVD $847, R12
	B    callbackasm1(SB)
	MOVD $848, R12
	B    callbackasm1(SB)
	MOVD $849, R12
	B    callbackasm1(SB)
	MOVD $850, R12
	B    callbackasm1(SB)
	MOVD $851, R12
	B    callbackasm1(SB)
	MOVD $852, R12
	B    callbackasm1(SB)
	MOVD $853, R12
	B    callbackasm1(SB)
	MOVD $854, R12
	B    callbackasm1(SB)
	MOVD $855, R12
	B    callbackasm1(SB)
	MOVD $856, R12
	B    callbackasm1(SB)
	MOVD $857, R12
	B    callbackasm1(SB)
	MOVD $858, R12
	B    callbackasm1(SB)
	MOVD $859, R12
	B    callbackasm1(SB)
	MOVD $860, R12
	B    callbackasm1(SB)
	MOVD $861, R12
	B    callbackasm1(SB)
	MOVD $862, R12
	B    callbackasm1(SB)
	MOVD $863, R12
	B    callbackasm1(SB)
	MOVD $864, R12
	B    callbackasm1(SB)
	MOVD $865, R12
	B    callbackasm1(SB)
	MOVD $866, R12
	B    callbackasm1(SB)
	MOVD $867, R12
	B    callbackasm1(SB)
	MOVD $868, R12
	B    callbackasm1(SB)
	MOVD $869, R12
	B    callbackasm1(SB)
	MOVD $870, R12
	B    callbackasm1(SB)
	MOVD $871, R12
	B    callbackasm1(SB)
	MOVD $872, R12
	B    callbackasm1(SB)
	MOVD $873, R12
	B    callbackasm1(SB)
	MOVD $874, R12
	B    callbackasm1(SB)
	MOVD $875, R12
	B    callbackasm1(SB)
	MOVD $876, R12
	B    callbackasm1(SB)
	MOVD $877, R12
	B    callbackasm1(SB)
	MOVD $878, R12
	B    callbackasm1(SB)
	MOVD $879, R12
	B    callbackasm1(SB)
	MOVD $880, R12
	B    callbackasm1(SB)
	MOVD $881, R12
	B    callbackasm1(SB)
	MOVD $882, R12
	B    callbackasm1(SB)
	MOVD $883, R12
	B    callbackasm1(SB)
	MOVD $884, R12
	B    callbackasm1(SB)
	MOVD $885, R12
	B    callbackasm1(SB)
	MOVD $886, R12
	B    callbackasm1(SB)
	MOVD $887, R12
	B    callbackasm1(SB)
	MOVD $888, R12
	B    callbackasm1(SB)
	MOVD $889, R12
	B    callbackasm1(SB)
	MOVD $890, R12
	B    callbackasm1(SB)
	MOVD $891, R12
	B    callbackasm1(SB)
	MOVD $892, R12
	B    callbackasm1(SB)
	MOVD $893, R12
	B    callbackasm1(SB)
	MOVD $894, R12
	B    callbackasm1(SB)
	MOVD $895, R12
	B    callbackasm1(SB)
	MOVD $896, R12
	B    callbackasm1(SB)
	MOVD $897, R12
	B    callbackasm1(SB)
	MOVD $898, R12
	B    callbackasm1(SB)
	MOVD $899, R12
	B    callbackasm1(SB)
	MOVD $900, R12
	B    callbackasm1(SB)
	MOVD $901, R12
	B    callbackasm1(SB)
	MOVD $902, R12
	B    callbackasm1(SB)
	MOVD $903, R12
	B    callbackasm1(SB)
	MOVD $904, R12
	B    callbackasm1(SB)
	MOVD $905, R12
	B    callbackasm1(SB)
	MOVD $906, R12
	B    callbackasm1(SB)
	MOVD $907, R12
	B    callbackasm1(SB)
	MOVD $908, R12
	B    callbackasm1(SB)
	MOVD $909, R12
	B    callbackasm1(SB)
	MOVD $910, R12
	B    callbackasm1(SB)
	MOVD $911, R12
	B    callbackasm1(SB)
	MOVD $912, R12
	B    callbackasm1(SB)
	MOVD $913, R12
	B    callbackasm1(SB)
	MOVD $914, R12
	B    callbackasm1(SB)
	MOVD $915, R12
	B    callbackasm1(SB)
	MOVD $916, R12
	B    callbackasm1(SB)
	MOVD $917, R12
	B    callbackasm1(SB)
	MOVD $918, R12
	B    callbackasm1(SB)
	MOVD $919, R12
	B    callbackasm1(SB)
	MOVD $920, R12
	B    callbackasm1(SB)
	MOVD $921, R12
	B    callbackasm1(SB)
	MOVD $922, R12
	B    callbackasm1(SB)
	MOVD $923, R12
	B    callbackasm1(SB)
	MOVD $924, R12
	B    callbackasm1(SB)
	MOVD $925, R12
	B    callbackasm1(SB)
	MOVD $926, R12
	B    callbackasm1(SB)
	MOVD $927, R12
	B    callbackasm1(SB)
	MOVD $928, R12
	B    callbackasm1(SB)
	MOVD $929, R12
	B    callbackasm1(SB)
	MOVD $930, R12
	B    callbackasm1(SB)
	MOVD $931, R12
	B    callbackasm1(SB)
	MOVD $932, R12
	B    callbackasm1(SB)
	MOVD $933, R12
	B    callbackasm1(SB)
	MOVD $934, R12
	B    callbackasm1(SB)
	MOVD $935, R12
	B    callbackasm1(SB)
	MOVD $936, R12
	B    callbackasm1(SB)
	MOVD $937, R12
	B    callbackasm1(SB)
	MOVD $938, R12
	B    callbackasm1(SB)
	MOVD $939, R12
	B    callbackasm1(SB)
	MOVD $940, R12
	B    callbackasm1(SB)
	MOVD $941, R12
	B    callbackasm1(SB)
	MOVD $942, R12
	B    callbackasm1(SB)
	MOVD $943, R12
	B    callbackasm1(SB)
	MOVD $944, R12
	B    callbackasm1(SB)
	MOVD $945, R12
	B    callbackasm1(SB)
	MOVD $946, R12
	B    callbackasm1(SB)
	MOVD $947, R12
	B    callbackasm1(SB)
	MOVD $948, R12
	B    callbackasm1(SB)
	MOVD $949, R12
	B    callbackasm1(SB)
	MOVD $950, R12
	B    callbackasm1(SB)
	MOVD $951, R12
	B    callbackasm1(SB)
	MOVD $952, R12
	B    callbackasm1(SB)
	MOVD $953, R12
	B    callbackasm1(SB)
	MOVD $954, R12
	B    callbackasm1(SB)
	MOVD $955, R12
	B    callbackasm1(SB)
	MOVD $956, R12
	B    callbackasm1(SB)
	MOVD $957, R12
	B    callbackasm1(SB)
	MOVD $958, R12
	B    callbackasm1(SB)
	MOVD $959, R12
	B    callbackasm1(SB)
	MOVD $960, R12
	B    callbackasm1(SB)
	MOVD $961, R12
	B    callbackasm1(SB)
	MOVD $962, R12
	B    callbackasm1(SB)
	MOVD $963, R12
	B    callbackasm1(SB)
	MOVD $964, R12
	B    callbackasm1(SB)
	MOVD $965, R12
	B    callbackasm1(SB)
	MOVD $966, R12
	B    callbackasm1(SB)
	MOVD $967, R12
	B    callbackasm1(SB)
	MOVD $968, R12
	B    callbackasm1(SB)
	MOVD $969, R12
	B    callbackasm1(SB)
	MOVD $970, R12
	B    callbackasm1(SB)
	MOVD $971, R12
	B    callbackasm1(SB)
	MOVD $972, R12
	B    callbackasm1(SB)
	MOVD $973, R12
	B    callbackasm1(SB)
	MOVD $974, R12
	B    callbackasm1(SB)
	MOVD $975, R12
	B    callbackasm1(SB)
	MOVD $976, R12
	B    callbackasm1(SB)
	MOVD $977, R12
	B    callbackasm1(SB)
	MOVD $978, R12
	B    callbackasm1(SB)
	MOVD $979, R12
	B    callbackasm1(SB)
	MOVD $980, R12
	B    callbackasm1(SB)
	MOVD $981, R12
	B    callbackasm1(SB)
	MOVD $982, R12
	B    callbackasm1(SB)
	MOVD $983, R12
	B    callbackasm1(SB)
	MOVD $984, R12
	B    callbackasm1(SB)
	MOVD $985, R12
	B    callbackasm1(SB)
	MOVD $986, R12
	B    callbackasm1(SB)
	MOVD $987, R12
	B    callbackasm1(SB)
	MOVD $988, R12
	B    callbackasm1(SB)
	MOVD $989, R12
	B    callbackasm1(SB)
	MOVD $990, R12
	B    callbackasm1(SB)
	MOVD $991, R12
	B    callbackasm1(SB)
	MOVD $992, R12
	B    callbackasm1(SB)
	MOVD $993, R12
	B    callbackasm1(SB)
	MOVD $994, R12
	B    callbackasm1(SB)
	MOVD $995, R12
	B    callbackasm1(SB)
	MOVD $996, R12
	B    callbackasm1(SB)
	MOVD $997, R12
	B    callbackasm1(SB)
	MOVD $998, R12
	B    callbackasm1(SB)
	MOVD $999, R12
	B    callbackasm1(SB)
	MOVD $1000, R12
	B    callbackasm1(SB)
	MOVD $1001, R12
	B    callbackasm1(SB)
	MOVD $1002, R12
	B    callbackasm1(SB)
	MOVD $1003, R12
	B    callbackasm1(SB)
	MOVD $1004, R12
	B    callbackasm1(SB)
	MOVD $1005, R12
	B    callbackasm1(SB)
	MOVD $1006, R12
	B    callbackasm1(SB)
	MOVD $1007, R12
	B    callbackasm1(SB)
	MOVD $1008, R12
	B    callbackasm1(SB)
	MOVD $1009, R12
	B    callbackasm1(SB)
	MOVD $1010, R12
	B    callbackasm1(SB)
	MOVD $1011, R12
	B    callbackasm1(SB)
	MOVD $1012, R12
	B    callbackasm1(SB)
	MOVD $1013, R12
	B    callbackasm1(SB)
	MOVD $1014, R12
	B    callbackasm1(SB)
	MOVD $1015, R12
	B    callbackasm1(SB)
	MOVD $1016, R12
	B    callbackasm1(SB)
	MOVD $1017, R12
	B    callbackasm1(SB)
	MOVD $1018, R12
	B    callbackasm1(SB)
	MOVD $1019, R12
	B    callbackasm1(SB)
	MOVD $1020, R12
	B    callbackasm1(SB)
	MOVD $1021, R12
	B    callbackasm1(SB)
	MOVD $1022, R12
	B    callbackasm1(SB)
	MOVD $1023, R12
	B    callbackasm1(SB)
	MOVD $1024, R12
	B    callbackasm1(SB)
	MOVD $1025, R12
	B    callbackasm1(SB)
	MOVD $1026, R12
	B    callbackasm1(SB)
	MOVD $1027, R12
	B    callbackasm1(SB)
	MOVD $1028, R12
	B    callbackasm1(SB)
	MOVD $1029, R12
	B    callbackasm1(SB)
	MOVD $1030, R12
	B    callbackasm1(SB)
	MOVD $1031, R12
	B    callbackasm1(SB)
	MOVD $1032, R12
	B    callbackasm1(SB)
	MOVD $1033, R12
	B    callbackasm1(SB)
	MOVD $1034, R12
	B    callbackasm1(SB)
	MOVD $1035, R12
	B    callbackasm1(SB)
	MOVD $1036, R12
	B    callbackasm1(SB)
	MOVD $1037, R12
	B    callbackasm1(SB)
	MOVD $1038, R12
	B    callbackasm1(SB)
	MOVD $1039, R12
	B    callbackasm1(SB)
	MOVD $1040, R12
	B    callbackasm1(SB)
	MOVD $1041, R12
	B    callbackasm1(SB)
	MOVD $1042, R12
	B    callbackasm1(SB)
	MOVD $1043, R12
	B    callbackasm1(SB)
	MOVD $1044, R12
	B    callbackasm1(SB)
	MOVD $1045, R12
	B    callbackasm1(SB)
	MOVD $1046, R12
	B    callbackasm1(SB)
	MOVD $1047, R12
	B    callbackasm1(SB)
	MOVD $1048, R12
	B    callbackasm1(SB)
	MOVD $1049, R12
	B    callbackasm1(SB)
	MOVD $1050, R12
	B    callbackasm1(SB)
	MOVD $1051, R12
	B    callbackasm1(SB)
	MOVD $1052, R12
	B    callbackasm1(SB)
	MOVD $1053, R12
	B    callbackasm1(SB)
	MOVD $1054, R12
	B    callbackasm1(SB)
	MOVD $1055, R12
	B    callbackasm1(SB)
	MOVD $1056, R12
	B    callbackasm1(SB)
	MOVD $1057, R12
	B    callbackasm1(SB)
	MOVD $1058, R12
	B    callbackasm1(SB)
	MOVD $1059, R12
	B    callbackasm1(SB)
	MOVD $1060, R12
	B    callbackasm1(SB)
	MOVD $1061, R12
	B    callbackasm1(SB)
	MOVD $1062, R12
	B    callbackasm1(SB)
	MOVD $1063, R12
	B    callbackasm1(SB)
	MOVD $1064, R12
	B    callbackasm1(SB)
	MOVD $1065, R12
	B    callbackasm1(SB)
	MOVD $1066, R12
	B    callbackasm1(SB)
	MOVD $1067, R12
	B    callbackasm1(SB)
	MOVD $1068, R12
	B    callbackasm1(SB)
	MOVD $1069, R12
	B    callbackasm1(SB)
	MOVD $1070, R12
	B    callbackasm1(SB)
	MOVD $1071, R12
	B    callbackasm1(SB)
	MOVD $1072, R12
	B    callbackasm1(SB)
	MOVD $1073, R12
	B    callbackasm1(SB)
	MOVD $1074, R12
	B    callbackasm1(SB)
	MOVD $1075, R12
	B    callbackasm1(SB)
	MOVD $1076, R12
	B    callbackasm1(SB)
	MOVD $1077, R12
	B    callbackasm1(SB)
	MOVD $1078, R12
	B    callbackasm1(SB)
	MOVD $1079, R12
	B    callbackasm1(SB)
	MOVD $1080, R12
	B    callbackasm1(SB)
	MOVD $1081, R12
	B    callbackasm1(SB)
	MOVD $1082, R12
	B    callbackasm1(SB)
	MOVD $1083, R12
	B    callbackasm1(SB)
	MOVD $1084, R12
	B    callbackasm1(SB)
	MOVD $1085, R12
	B    callbackasm1(SB)
	MOVD $1086, R12
	B    callbackasm1(SB)
	MOVD $1087, R12
	B    callbackasm1(SB)
	MOVD $1088, R12
	B    callbackasm1(SB)
	MOVD $1089, R12
	B    callbackasm1(SB)
	MOVD $1090, R12
	B    callbackasm1(SB)
	MOVD $1091, R12
	B    callbackasm1(SB)
	MOVD $1092, R12
	B    callbackasm1(SB)
	MOVD $1093, R12
	B    callbackasm1(SB)
	MOVD $1094, R12
	B    callbackasm1(SB)
	MOVD $1095, R12
	B    callbackasm1(SB)
	MOVD $1096, R12
	B    callbackasm1(SB)
	MOVD $1097, R12
	B    callbackasm1(SB)
	MOVD $1098, R12
	B    callbackasm1(SB)
	MOVD $1099, R12
	B    callbackasm1(SB)
	MOVD $1100, R12
	B    callbackasm1(SB)
	MOVD $1101, R12
	B    callbackasm1(SB)
	MOVD $1102, R12
	B    callbackasm1(SB)
	MOVD $1103, R12
	B    callbackasm1(SB)
	MOVD $1104, R12
	B    callbackasm1(SB)
	MOVD $1105, R12
	B    callbackasm1(SB)
	MOVD $1106, R12
	B    callbackasm1(SB)
	MOVD $1107, R12
	B    callbackasm1(SB)
	MOVD $1108, R12
	B    callbackasm1(SB)
	MOVD $1109, R12
	B    callbackasm1(SB)
	MOVD $1110, R12
	B    callbackasm1(SB)
	MOVD $1111, R12
	B    callbackasm1(SB)
	MOVD $1112, R12
	B    callbackasm1(SB)
	MOVD $1113, R12
	B    callbackasm1(SB)
	MOVD $1114, R12
	B    callbackasm1(SB)
	MOVD $1115, R12
	B    callbackasm1(SB)
	MOVD $1116, R12
	B    callbackasm1(SB)
	MOVD $1117, R12
	B    callbackasm1(SB)
	MOVD $1118, R12
	B    callbackasm1(SB)
	MOVD $1119, R12
	B    callbackasm1(SB)
	MOVD $1120, R12
	B    callbackasm1(SB)
	MOVD $1121, R12
	B    callbackasm1(SB)
	MOVD $1122, R12
	B    callbackasm1(SB)
	MOVD $1123, R12
	B    callbackasm1(SB)
	MOVD $1124, R12
	B    callbackasm1(SB)
	MOVD $1125, R12
	B    callbackasm1(SB)
	MOVD $1126, R12
	B    callbackasm1(SB)
	MOVD $1127, R12
	B    callbackasm1(SB)
	MOVD $1128, R12
	B    callbackasm1(SB)
	MOVD $1129, R12
	B    callbackasm1(SB)
	MOVD $1130, R12
	B    callbackasm1(SB)
	MOVD $1131, R12
	B    callbackasm1(SB)
	MOVD $1132, R12
	B    callbackasm1(SB)
	MOVD $1133, R12
	B    callbackasm1(SB)
	MOVD $1134, R12
	B    callbackasm1(SB)
	MOVD $1135, R12
	B    callbackasm1(SB)
	MOVD $1136, R12
	B    callbackasm1(SB)
	MOVD $1137, R12
	B    callbackasm1(SB)
	MOVD $1138, R12
	B    callbackasm1(SB)
	MOVD $1139, R12
	B    callbackasm1(SB)
	MOVD $1140, R12
	B    callbackasm1(SB)
	MOVD $1141, R12
	B    callbackasm1(SB)
	MOVD $1142, R12
	B    callbackasm1(SB)
	MOVD $1143, R12
	B    callbackasm1(SB)
	MOVD $1144, R12
	B    callbackasm1(SB)
	MOVD $1145, R12
	B    callbackasm1(SB)
	MOVD $1146, R12
	B    callbackasm1(SB)
	MOVD $1147, R12
	B    callbackasm1(SB)
	MOVD $1148, R12
	B    callbackasm1(SB)
	MOVD $1149, R12
	B    callbackasm1(SB)
	MOVD $1150, R12
	B    callbackasm1(SB)
	MOVD $1151, R12
	B    callbackasm1(SB)
	MOVD $1152, R12
	B    callbackasm1(SB)
	MOVD $1153, R12
	B    callbackasm1(SB)
	MOVD $1154, R12
	B    callbackasm1(SB)
	MOVD $1155, R12
	B    callbackasm1(SB)
	MOVD $1156, R12
	B    callbackasm1(SB)
	MOVD $1157, R12
	B    callbackasm1(SB)
	MOVD $1158, R12
	B    callbackasm1(SB)
	MOVD $1159, R12
	B    callbackasm1(SB)
	MOVD $1160, R12
	B    callbackasm1(SB)
	MOVD $1161, R12
	B    callbackasm1(SB)
	MOVD $1162, R12
	B    callbackasm1(SB)
	MOVD $1163, R12
	B    callbackasm1(SB)
	MOVD $1164, R12
	B    callbackasm1(SB)
	MOVD $1165, R12
	B    callbackasm1(SB)
	MOVD $1166, R12
	B    callbackasm1(SB)
	MOVD $1167, R12
	B    callbackasm1(SB)
	MOVD $1168, R12
	B    callbackasm1(SB)
	MOVD $1169, R12
	B    callbackasm1(SB)
	MOVD $1170, R12
	B    callbackasm1(SB)
	MOVD $1171, R12
	B    callbackasm1(SB)
	MOVD $1172, R12
	B    callbackasm1(SB)
	MOVD $1173, R12
	B    callbackasm1(SB)
	MOVD $1174, R12
	B    callbackasm1(SB)
	MOVD $1175, R12
	B    callbackasm1(SB)
	MOVD $1176, R12
	B    callbackasm1(SB)
	MOVD $1177, R12
	B    callbackasm1(SB)
	MOVD $1178, R12
	B    callbackasm1(SB)
	MOVD $1179, R12
	B    callbackasm1(SB)
	MOVD $1180, R12
	B    callbackasm1(SB)
	MOVD $1181, R12
	B    callbackasm1(SB)
	MOVD $1182, R12
	B    callbackasm1(SB)
	MOVD $1183, R12
	B    callbackasm1(SB)
	MOVD $1184, R12
	B    callbackasm1(SB)
	MOVD $1185, R12
	B    callbackasm1(SB)
	MOVD $1186, R12
	B    callbackasm1(SB)
	MOVD $1187, R12
	B    callbackasm1(SB)
	MOVD $1188, R12
	B    callbackasm1(SB)
	MOVD $1189, R12
	B    callbackasm1(SB)
	MOVD $1190, R12
	B    callbackasm1(SB)
	MOVD $1191, R12
	B    callbackasm1(SB)
	MOVD $1192, R12
	B    callbackasm1(SB)
	MOVD $1193, R12
	B    callbackasm1(SB)
	MOVD $1194, R12
	B    callbackasm1(SB)
	MOVD $1195, R12
	B    callbackasm1(SB)
	MOVD $1196, R12
	B    callbackasm1(SB)
	MOVD $1197, R12
	B    callbackasm1(SB)
	MOVD $1198, R12
	B    callbackasm1(SB)
	MOVD $1199, R12
	B    callbackasm1(SB)
	MOVD $1200, R12
	B    callbackasm1(SB)
	MOVD $1201, R12
	B    callbackasm1(SB)
	MOVD $1202, R12
	B    callbackasm1(SB)
	MOVD $1203, R12
	B    callbackasm1(SB)
	MOVD $1204, R12
	B    callbackasm1(SB)
	MOVD $1205, R12
	B    callbackasm1(SB)
	MOVD $1206, R12
	B    callbackasm1(SB)
	MOVD $1207, R12
	B    callbackasm1(SB)
	MOVD $1208, R12
	B    callbackasm1(SB)
	MOVD $1209, R12
	B    callbackasm1(SB)
	MOVD $1210, R12
	B    callbackasm1(SB)
	MOVD $1211, R12
	B    callbackasm1(SB)
	MOVD $1212, R12
	B    callbackasm1(SB)
	MOVD $1213, R12
	B    callbackasm1(SB)
	MOVD $1214, R12
	B    callbackasm1(SB)
	MOVD $1215, R12
	B    callbackasm1(SB)
	MOVD $1216, R12
	B    callbackasm1(SB)
	MOVD $1217, R12
	B    callbackasm1(SB)
	MOVD $1218, R12
	B    callbackasm1(SB)
	MOVD $1219, R12
	B    callbackasm1(SB)
	MOVD $1220, R12
	B    callbackasm1(SB)
	MOVD $1221, R12
	B    callbackasm1(SB)
	MOVD $1222, R12
	B    callbackasm1(SB)
	MOVD $1223, R12
	B    callbackasm1(SB)
	MOVD $1224, R12
	B    callbackasm1(SB)
	MOVD $1225, R12
	B    callbackasm1(SB)
	MOVD $1226, R12
	B    callbackasm1(SB)
	MOVD $1227, R12
	B    callbackasm1(SB)
	MOVD $1228, R12
	B    callbackasm1(SB)
	MOVD $1229, R12
	B    callbackasm1(SB)
	MOVD $1230, R12
	B    callbackasm1(SB)
	MOVD $1231, R12
	B    callbackasm1(SB)
	MOVD $1232, R12
	B    callbackasm1(SB)
	MOVD $1233, R12
	B    callbackasm1(SB)
	MOVD $1234, R12
	B    callbackasm1(SB)
	MOVD $1235, R12
	B    callbackasm1(SB)
	MOVD $1236, R12
	B    callbackasm1(SB)
	MOVD $1237, R12
	B    callbackasm1(SB)
	MOVD $1238, R12
	B    callbackasm1(SB)
	MOVD $1239, R12
	B    callbackasm1(SB)
	MOVD $1240, R12
	B    callbackasm1(SB)
	MOVD $1241, R12
	B    callbackasm1(SB)
	MOVD $1242, R12
	B    callbackasm1(SB)
	MOVD $1243, R12
	B    callbackasm1(SB)
	MOVD $1244, R12
	B    callbackasm1(SB)
	MOVD $1245, R12
	B    callbackasm1(SB)
	MOVD $1246, R12
	B    callbackasm1(SB)
	MOVD $1247, R12
	B    callbackasm1(SB)
	MOVD $1248, R12
	B    callbackasm1(SB)
	MOVD $1249, R12
	B    callbackasm1(SB)
	MOVD $1250, R12
	B    callbackasm1(SB)
	MOVD $1251, R12
	B    callbackasm1(SB)
	MOVD $1252, R12
	B    callbackasm1(SB)
	MOVD $1253, R12
	B    callbackasm1(SB)
	MOVD $1254, R12
	B    callbackasm1(SB)
	MOVD $1255, R12
	B    callbackasm1(SB)
	MOVD $1256, R12
	B    callbackasm1(SB)
	MOVD $1257, R12
	B    callbackasm1(SB)
	MOVD $1258, R12
	B    callbackasm1(SB)
	MOVD $1259, R12
	B    callbackasm1(SB)
	MOVD $1260, R12
	B    callbackasm1(SB)
	MOVD $1261, R12
	B    callbackasm1(SB)
	MOVD $1262, R12
	B    callbackasm1(SB)
	MOVD $1263, R12
	B    callbackasm1(SB)
	MOVD $1264, R12
	B    callbackasm1(SB)
	MOVD $1265, R12
	B    callbackasm1(SB)
	MOVD $1266, R12
	B    callbackasm1(SB)
	MOVD $1267, R12
	B    callbackasm1(SB)
	MOVD $1268, R12
	B    callbackasm1(SB)
	MOVD $1269, R12
	B    callbackasm1(SB)
	MOVD $1270, R12
	B    callbackasm1(SB)
	MOVD $1271, R12
	B    callbackasm1(SB)
	MOVD $1272, R12
	B    callbackasm1(SB)
	MOVD $1273, R12
	B    callbackasm1(SB)
	MOVD $1274, R12
	B    callbackasm1(SB)
	MOVD $1275, R12
	B    callbackasm1(SB)
	MOVD $1276, R12
	B    callbackasm1(SB)
	MOVD $1277, R12
	B    callbackasm1(SB)
	MOVD $1278, R12
	B    callbackasm1(SB)
	MOVD $1279, R12
	B    callbackasm1(SB)
	MOVD $1280, R12
	B    callbackasm1(SB)
	MOVD $1281, R12
	B    callbackasm1(SB)
	MOVD $1282, R12
	B    callbackasm1(SB)
	MOVD $1283, R12
	B    callbackasm1(SB)
	MOVD $1284, R12
	B    callbackasm1(SB)
	MOVD $1285, R12
	B    callbackasm1(SB)
	MOVD $1286, R12
	B    callbackasm1(SB)
	MOVD $1287, R12
	B    callbackasm1(SB)
	MOVD $1288, R12
	B    callbackasm1(SB)
	MOVD $1289, R12
	B    callbackasm1(SB)
	MOVD $1290, R12
	B    callbackasm1(SB)
	MOVD $1291, R12
	B    callbackasm1(SB)
	MOVD $1292, R12
	B    callbackasm1(SB)
	MOVD $1293, R12
	B    callbackasm1(SB)
	MOVD $1294, R12
	B    callbackasm1(SB)
	MOVD $1295, R12
	B    callbackasm1(SB)
	MOVD $1296, R12
	B    callbackasm1(SB)
	MOVD $1297, R12
	B    callbackasm1(SB)
	MOVD $1298, R12
	B    callbackasm1(SB)
	MOVD $1299, R12
	B    callbackasm1(SB)
	MOVD $1300, R12
	B    callbackasm1(SB)
	MOVD $1301, R12
	B    callbackasm1(SB)
	MOVD $1302, R12
	B    callbackasm1(SB)
	MOVD $1303, R12
	B    callbackasm1(SB)
	MOVD $1304, R12
	B    callbackasm1(SB)
	MOVD $1305, R12
	B    callbackasm1(SB)
	MOVD $1306, R12
	B    callbackasm1(SB)
	MOVD $1307, R12
	B    callbackasm1(SB)
	MOVD $1308, R12
	B    callbackasm1(SB)
	MOVD $1309, R12
	B    callbackasm1(SB)
	MOVD $1310, R12
	B    callbackasm1(SB)
	MOVD $1311, R12
	B    callbackasm1(SB)
	MOVD $1312, R12
	B    callbackasm1(SB)
	MOVD $1313, R12
	B    callbackasm1(SB)
	MOVD $1314, R12
	B    callbackasm1(SB)
	MOVD $1315, R12
	B    callbackasm1(SB)
	MOVD $1316, R12
	B    callbackasm1(SB)
	MOVD $1317, R12
	B    callbackasm1(SB)
	MOVD $1318, R12
	B    callbackasm1(SB)
	MOVD $1319, R12
	B    callbackasm1(SB)
	MOVD $1320, R12
	B    callbackasm1(SB)
	MOVD $1321, R12
	B    callbackasm1(SB)
	MOVD $1322, R12
	B    callbackasm1(SB)
	MOVD $1323, R12
	B    callbackasm1(SB)
	MOVD $1324, R12
	B    callbackasm1(SB)
	MOVD $1325, R12
	B    callbackasm1(SB)
	MOVD $1326, R12
	B    callbackasm1(SB)
	MOVD $1327, R12
	B    callbackasm1(SB)
	MOVD $1328, R12
	B    callbackasm1(SB)
	MOVD $1329, R12
	B    callbackasm1(SB)
	MOVD $1330, R12
	B    callbackasm1(SB)
	MOVD $1331, R12
	B    callbackasm1(SB)
	MOVD $1332, R12
	B    callbackasm1(SB)
	MOVD $1333, R12
	B    callbackasm1(SB)
	MOVD $1334, R12
	B    callbackasm1(SB)
	MOVD $1335, R12
	B    callbackasm1(SB)
	MOVD $1336, R12
	B    callbackasm1(SB)
	MOVD $1337, R12
	B    callbackasm1(SB)
	MOVD $1338, R12
	B    callbackasm1(SB)
	MOVD $1339, R12
	B    callbackasm1(SB)
	MOVD $1340, R12
	B    callbackasm1(SB)
	MOVD $1341, R12
	B    callbackasm1(SB)
	MOVD $1342, R12
	B    callbackasm1(SB)
	MOVD $1343, R12
	B    callbackasm1(SB)
	MOVD $1344, R12
	B    callbackasm1(SB)
	MOVD $1345, R12
	B    callbackasm1(SB)
	MOVD $1346, R12
	B    callbackasm1(SB)
	MOVD $1347, R12
	B    callbackasm1(SB)
	MOVD $1348, R12
	B    callbackasm1(SB)
	MOVD $1349, R12
	B    callbackasm1(SB)
	MOVD $1350, R12
	B    callbackasm1(SB)
	MOVD $1351, R12
	B    callbackasm1(SB)
	MOVD $1352, R12
	B    callbackasm1(SB)
	MOVD $1353, R12
	B    callbackasm1(SB)
	MOVD $1354, R12
	B    callbackasm1(SB)
	MOVD $1355, R12
	B    callbackasm1(SB)
	MOVD $1356, R12
	B    callbackasm1(SB)
	MOVD $1357, R12
	B    callbackasm1(SB)
	MOVD $1358, R12
	B    callbackasm1(SB)
	MOVD $1359, R12
	B    callbackasm1(SB)
	MOVD $1360, R12
	B    callbackasm1(SB)
	MOVD $1361, R12
	B    callbackasm1(SB)
	MOVD $1362, R12
	B    callbackasm1(SB)
	MOVD $1363, R12
	B    callbackasm1(SB)
	MOVD $1364, R12
	B    callbackasm1(SB)
	MOVD $1365, R12
	B    callbackasm1(SB)
	MOVD $1366, R12
	B    callbackasm1(SB)
	MOVD $1367, R12
	B    callbackasm1(SB)
	MOVD $1368, R12
	B    callbackasm1(SB)
	MOVD $1369, R12
	B    callbackasm1(SB)
	MOVD $1370, R12
	B    callbackasm1(SB)
	MOVD $1371, R12
	B    callbackasm1(SB)
	MOVD $1372, R12
	B    callbackasm1(SB)
	MOVD $1373, R12
	B    callbackasm1(SB)
	MOVD $1374, R12
	B    callbackasm1(SB)
	MOVD $1375, R12
	B    callbackasm1(SB)
	MOVD $1376, R12
	B    callbackasm1(SB)
	MOVD $1377, R12
	B    callbackasm1(SB)
	MOVD $1378, R12
	B    callbackasm1(SB)
	MOVD $1379, R12
	B    callbackasm1(SB)
	MOVD $1380, R12
	B    callbackasm1(SB)
	MOVD $1381, R12
	B    callbackasm1(SB)
	MOVD $1382, R12
	B    callbackasm1(SB)
	MOVD $1383, R12
	B    callbackasm1(SB)
	MOVD $1384, R12
	B    callbackasm1(SB)
	MOVD $1385, R12
	B    callbackasm1(SB)
	MOVD $1386, R12
	B    callbackasm1(SB)
	MOVD $1387, R12
	B    callbackasm1(SB)
	MOVD $1388, R12
	B    callbackasm1(SB)
	MOVD $1389, R12
	B    callbackasm1(SB)
	MOVD $1390, R12
	B    callbackasm1(SB)
	MOVD $1391, R12
	B    callbackasm1(SB)
	MOVD $1392, R12
	B    callbackasm1(SB)
	MOVD $1393, R12
	B    callbackasm1(SB)
	MOVD $1394, R12
	B    callbackasm1(SB)
	MOVD $1395, R12
	B    callbackasm1(SB)
	MOVD $1396, R12
	B    callbackasm1(SB)
	MOVD $1397, R12
	B    callbackasm1(SB)
	MOVD $1398, R12
	B    callbackasm1(SB)
	MOVD $1399, R12
	B    callbackasm1(SB)
	MOVD $1400, R12
	B    callbackasm1(SB)
	MOVD $1401, R12
	B    callbackasm1(SB)
	MOVD $1402, R12
	B    callbackasm1(SB)
	MOVD $1403, R12
	B    callbackasm1(SB)
	MOVD $1404, R12
	B    callbackasm1(SB)
	MOVD $1405, R12
	B    callbackasm1(SB)
	MOVD $1406, R12
	B    callbackasm1(SB)
	MOVD $1407, R12
	B    callbackasm1(SB)
	MOVD $1408, R12
	B    callbackasm1(SB)
	MOVD $1409, R12
	B    callbackasm1(SB)
	MOVD $1410, R12
	B    callbackasm1(SB)
	MOVD $1411, R12
	B    callbackasm1(SB)
	MOVD $1412, R12
	B    callbackasm1(SB)
	MOVD $1413, R12
	B    callbackasm1(SB)
	MOVD $1414, R12
	B    callbackasm1(SB)
	MOVD $1415, R12
	B    callbackasm1(SB)
	MOVD $1416, R12
	B    callbackasm1(SB)
	MOVD $1417, R12
	B    callbackasm1(SB)
	MOVD $1418, R12
	B    callbackasm1(SB)
	MOVD $1419, R12
	B    callbackasm1(SB)
	MOVD $1420, R12
	B    callbackasm1(SB)
	MOVD $1421, R12
	B    callbackasm1(SB)
	MOVD $1422, R12
	B    callbackasm1(SB)
	MOVD $1423, R12
	B    callbackasm1(SB)
	MOVD $1424, R12
	B    callbackasm1(SB)
	MOVD $1425, R12
	B    callbackasm1(SB)
	MOVD $1426, R12
	B    callbackasm1(SB)
	MOVD $1427, R12
	B    callbackasm1(SB)
	MOVD $1428, R12
	B    callbackasm1(SB)
	MOVD $1429, R12
	B    callbackasm1(SB)
	MOVD $1430, R12
	B    callbackasm1(SB)
	MOVD $1431, R12
	B    callbackasm1(SB)
	MOVD $1432, R12
	B    callbackasm1(SB)
	MOVD $1433, R12
	B    callbackasm1(SB)
	MOVD $1434, R12
	B    callbackasm1(SB)
	MOVD $1435, R12
	B    callbackasm1(SB)
	MOVD $1436, R12
	B    callbackasm1(SB)
	MOVD $1437, R12
	B    callbackasm1(SB)
	MOVD $1438, R12
	B    callbackasm1(SB)
	MOVD $1439, R12
	B    callbackasm1(SB)
	MOVD $1440, R12
	B    callbackasm1(SB)
	MOVD $1441, R12
	B    callbackasm1(SB)
	MOVD $1442, R12
	B    callbackasm1(SB)
	MOVD $1443, R12
	B    callbackasm1(SB)
	MOVD $1444, R12
	B    callbackasm1(SB)
	MOVD $1445, R12
	B    callbackasm1(SB)
	MOVD $1446, R12
	B    callbackasm1(SB)
	MOVD $1447, R12
	B    callbackasm1(SB)
	MOVD $1448, R12
	B    callbackasm1(SB)
	MOVD $1449, R12
	B    callbackasm1(SB)
	MOVD $1450, R12
	B    callbackasm1(SB)
	MOVD $1451, R12
	B    callbackasm1(SB)
	MOVD $1452, R12
	B    callbackasm1(SB)
	MOVD $1453, R12
	B    callbackasm1(SB)
	MOVD $1454, R12
	B    callbackasm1(SB)
	MOVD $1455, R12
	B    callbackasm1(SB)
	MOVD $1456, R12
	B    callbackasm1(SB)
	MOVD $1457, R12
	B    callbackasm1(SB)
	MOVD $1458, R12
	B    callbackasm1(SB)
	MOVD $1459, R12
	B    callbackasm1(SB)
	MOVD $1460, R12
	B    callbackasm1(SB)
	MOVD $1461, R12
	B    callbackasm1(SB)
	MOVD $1462, R12
	B    callbackasm1(SB)
	MOVD $1463, R12
	B    callbackasm1(SB)
	MOVD $1464, R12
	B    callbackasm1(SB)
	MOVD $1465, R12
	B    callbackasm1(SB)
	MOVD $1466, R12
	B    callbackasm1(SB)
	MOVD $1467, R12
	B    callbackasm1(SB)
	MOVD $1468, R12
	B    callbackasm1(SB)
	MOVD $1469, R12
	B    callbackasm1(SB)
	MOVD $1470, R12
	B    callbackasm1(SB)
	MOVD $1471, R12
	B    callbackasm1(SB)
	MOVD $1472, R12
	B    callbackasm1(SB)
	MOVD $1473, R12
	B    callbackasm1(SB)
	MOVD $1474, R12
	B    callbackasm1(SB)
	MOVD $1475, R12
	B    callbackasm1(SB)
	MOVD $1476, R12
	B    callbackasm1(SB)
	MOVD $1477, R12
	B    callbackasm1(SB)
	MOVD $1478, R12
	B    callbackasm1(SB)
	MOVD $1479, R12
	B    callbackasm1(SB)
	MOVD $1480, R12
	B    callbackasm1(SB)
	MOVD $1481, R12
	B    callbackasm1(SB)
	MOVD $1482, R12
	B    callbackasm1(SB)
	MOVD $1483, R12
	B    callbackasm1(SB)
	MOVD $1484, R12
	B    callbackasm1(SB)
	MOVD $1485, R12
	B    callbackasm1(SB)
	MOVD $1486, R12
	B    callbackasm1(SB)
	MOVD $1487, R12
	B    callbackasm1(SB)
	MOVD $1488, R12
	B    callbackasm1(SB)
	MOVD $1489, R12
	B    callbackasm1(SB)
	MOVD $1490, R12
	B    callbackasm1(SB)
	MOVD $1491, R12
	B    callbackasm1(SB)
	MOVD $1492, R12
	B    callbackasm1(SB)
	MOVD $1493, R12
	B    callbackasm1(SB)
	MOVD $1494, R12
	B    callbackasm1(SB)
	MOVD $1495, R12
	B    callbackasm1(SB)
	MOVD $1496, R12
	B    callbackasm1(SB)
	MOVD $1497, R12
	B    callbackasm1(SB)
	MOVD $1498, R12
	B    callbackasm1(SB)
	MOVD $1499, R12
	B    callbackasm1(SB)
	MOVD $1500, R12
	B    callbackasm1(SB)
	MOVD $1501, R12
	B    callbackasm1(SB)
	MOVD $1502, R12
	B    callbackasm1(SB)
	MOVD $1503, R12
	B    callbackasm1(SB)
	MOVD $1504, R12
	B    callbackasm1(SB)
	MOVD $1505, R12
	B    callbackasm1(SB)
	MOVD $1506, R12
	B    callbackasm1(SB)
	MOVD $1507, R12
	B    callbackasm1(SB)
	MOVD $1508, R12
	B    callbackasm1(SB)
	MOVD $1509, R12
	B    callbackasm1(SB)
	MOVD $1510, R12
	B    callbackasm1(SB)
	MOVD $1511, R12
	B    callbackasm1(SB)
	MOVD $1512, R12
	B    callbackasm1(SB)
	MOVD $1513, R12
	B    callbackasm1(SB)
	MOVD $1514, R12
	B    callbackasm1(SB)
	MOVD $1515, R12
	B    callbackasm1(SB)
	MOVD $1516, R12
	B    callbackasm1(SB)
	MOVD $1517, R12
	B    callbackasm1(SB)
	MOVD $1518, R12
	B    callbackasm1(SB)
	MOVD $1519, R12
	B    callbackasm1(SB)
	MOVD $1520, R12
	B    callbackasm1(SB)
	MOVD $1521, R12
	B    callbackasm1(SB)
	MOVD $1522, R12
	B    callbackasm1(SB)
	MOVD $1523, R12
	B    callbackasm1(SB)
	MOVD $1524, R12
	B    callbackasm1(SB)
	MOVD $1525, R12
	B    callbackasm1(SB)
	MOVD $1526, R12
	B    callbackasm1(SB)
	MOVD $1527, R12
	B    callbackasm1(SB)
	MOVD $1528, R12
	B    callbackasm1(SB)
	MOVD $1529, R12
	B    callbackasm1(SB)
	MOVD $1530, R12
	B    callbackasm1(SB)
	MOVD $1531, R12
	B    callbackasm1(SB)
	MOVD $1532, R12
	B    callbackasm1(SB)
	MOVD $1533, R12
	B    callbackasm1(SB)
	MOVD $1534, R12
	B    callbackasm1(SB)
	MOVD $1535, R12
	B    callbackasm1(SB)
	MOVD $1536, R12
	B    callbackasm1(SB)
	MOVD $1537, R12
	B    callbackasm1(SB)
	MOVD $1538, R12
	B    callbackasm1(SB)
	MOVD $1539, R12
	B    callbackasm1(SB)
	MOVD $1540, R12
	B    callbackasm1(SB)
	MOVD $1541, R12
	B    callbackasm1(SB)
	MOVD $1542, R12
	B    callbackasm1(SB)
	MOVD $1543, R12
	B    callbackasm1(SB)
	MOVD $1544, R12
	B    callbackasm1(SB)
	MOVD $1545, R12
	B    callbackasm1(SB)
	MOVD $1546, R12
	B    callbackasm1(SB)
	MOVD $1547, R12
	B    callbackasm1(SB)
	MOVD $1548, R12
	B    callbackasm1(SB)
	MOVD $1549, R12
	B    callbackasm1(SB)
	MOVD $1550, R12
	B    callbackasm1(SB)
	MOVD $1551, R12
	B    callbackasm1(SB)
	MOVD $1552, R12
	B    callbackasm1(SB)
	MOVD $1553, R12
	B    callbackasm1(SB)
	MOVD $1554, R12
	B    callbackasm1(SB)
	MOVD $1555, R12
	B    callbackasm1(SB)
	MOVD $1556, R12
	B    callbackasm1(SB)
	MOVD $1557, R12
	B    callbackasm1(SB)
	MOVD $1558, R12
	B    callbackasm1(SB)
	MOVD $1559, R12
	B    callbackasm1(SB)
	MOVD $1560, R12
	B    callbackasm1(SB)
	MOVD $1561, R12
	B    callbackasm1(SB)
	MOVD $1562, R12
	B    callbackasm1(SB)
	MOVD $1563, R12
	B    callbackasm1(SB)
	MOVD $1564, R12
	B    callbackasm1(SB)
	MOVD $1565, R12
	B    callbackasm1(SB)
	MOVD $1566, R12
	B    callbackasm1(SB)
	MOVD $1567, R12
	B    callbackasm1(SB)
	MOVD $1568, R12
	B    callbackasm1(SB)
	MOVD $1569, R12
	B    callbackasm1(SB)
	MOVD $1570, R12
	B    callbackasm1(SB)
	MOVD $1571, R12
	B    callbackasm1(SB)
	MOVD $1572, R12
	B    callbackasm1(SB)
	MOVD $1573, R12
	B    callbackasm1(SB)
	MOVD $1574, R12
	B    callbackasm1(SB)
	MOVD $1575, R12
	B    callbackasm1(SB)
	MOVD $1576, R12
	B    callbackasm1(SB)
	MOVD $1577, R12
	B    callbackasm1(SB)
	MOVD $1578, R12
	B    callbackasm1(SB)
	MOVD $1579, R12
	B    callbackasm1(SB)
	MOVD $1580, R12
	B    callbackasm1(SB)
	MOVD $1581, R12
	B    callbackasm1(SB)
	MOVD $1582, R12
	B    callbackasm1(SB)
	MOVD $1583, R12
	B    callbackasm1(SB)
	MOVD $1584, R12
	B    callbackasm1(SB)
	MOVD $1585, R12
	B    callbackasm1(SB)
	MOVD $1586, R12
	B    callbackasm1(SB)
	MOVD $1587, R12
	B    callbackasm1(SB)
	MOVD $1588, R12
	B    callbackasm1(SB)
	MOVD $1589, R12
	B    callbackasm1(SB)
	MOVD $1590, R12
	B    callbackasm1(SB)
	MOVD $1591, R12
	B    callbackasm1(SB)
	MOVD $1592, R12
	B    callbackasm1(SB)
	MOVD $1593, R12
	B    callbackasm1(SB)
	MOVD $1594, R12
	B    callbackasm1(SB)
	MOVD $1595, R12
	B    callbackasm1(SB)
	MOVD $1596, R12
	B    callbackasm1(SB)
	MOVD $1597, R12
	B    callbackasm1(SB)
	MOVD $1598, R12
	B    callbackasm1(SB)
	MOVD $1599, R12
	B    callbackasm1(SB)
	MOVD $1600, R12
	B    callbackasm1(SB)
	MOVD $1601, R12
	B    callbackasm1(SB)
	MOVD $1602, R12
	B    callbackasm1(SB)
	MOVD $1603, R12
	B    callbackasm1(SB)
	MOVD $1604, R12
	B    callbackasm1(SB)
	MOVD $1605, R12
	B    callbackasm1(SB)
	MOVD $1606, R12
	B    callbackasm1(SB)
	MOVD $1607, R12
	B    callbackasm1(SB)
	MOVD $1608, R12
	B    callbackasm1(SB)
	MOVD $1609, R12
	B    callbackasm1(SB)
	MOVD $1610, R12
	B    callbackasm1(SB)
	MOVD $1611, R12
	B    callbackasm1(SB)
	MOVD $1612, R12
	B    callbackasm1(SB)
	MOVD $1613, R12
	B    callbackasm1(SB)
	MOVD $1614, R12
	B    callbackasm1(SB)
	MOVD $1615, R12
	B    callbackasm1(SB)
	MOVD $1616, R12
	B    callbackasm1(SB)
	MOVD $1617, R12
	B    callbackasm1(SB)
	MOVD $1618, R12
	B    callbackasm1(SB)
	MOVD $1619, R12
	B    callbackasm1(SB)
	MOVD $1620, R12
	B    callbackasm1(SB)
	MOVD $1621, R12
	B    callbackasm1(SB)
	MOVD $1622, R12
	B    callbackasm1(SB)
	MOVD $1623, R12
	B    callbackasm1(SB)
	MOVD $1624, R12
	B    callbackasm1(SB)
	MOVD $1625, R12
	B    callbackasm1(SB)
	MOVD $1626, R12
	B    callbackasm1(SB)
	MOVD $1627, R12
	B    callbackasm1(SB)
	MOVD $1628, R12
	B    callbackasm1(SB)
	MOVD $1629, R12
	B    callbackasm1(SB)
	MOVD $1630, R12
	B    callbackasm1(SB)
	MOVD $1631, R12
	B    callbackasm1(SB)
	MOVD $1632, R12
	B    callbackasm1(SB)
	MOVD $1633, R12
	B    callbackasm1(SB)
	MOVD $1634, R12
	B    callbackasm1(SB)
	MOVD $1635, R12
	B    callbackasm1(SB)
	MOVD $1636, R12
	B    callbackasm1(SB)
	MOVD $1637, R12
	B    callbackasm1(SB)
	MOVD $1638, R12
	B    callbackasm1(SB)
	MOVD $1639, R12
	B    callbackasm1(SB)
	MOVD $1640, R12
	B    callbackasm1(SB)
	MOVD $1641, R12
	B    callbackasm1(SB)
	MOVD $1642, R12
	B    callbackasm1(SB)
	MOVD $1643, R12
	B    callbackasm1(SB)
	MOVD $1644, R12
	B    callbackasm1(SB)
	MOVD $1645, R12
	B    callbackasm1(SB)
	MOVD $1646, R12
	B    callbackasm1(SB)
	MOVD $1647, R12
	B    callbackasm1(SB)
	MOVD $1648, R12
	B    callbackasm1(SB)
	MOVD $1649, R12
	B    callbackasm1(SB)
	MOVD $1650, R12
	B    callbackasm1(SB)
	MOVD $1651, R12
	B    callbackasm1(SB)
	MOVD $1652, R12
	B    callbackasm1(SB)
	MOVD $1653, R12
	B    callbackasm1(SB)
	MOVD $1654, R12
	B    callbackasm1(SB)
	MOVD $1655, R12
	B    callbackasm1(SB)
	MOVD $1656, R12
	B    callbackasm1(SB)
	MOVD $1657, R12
	B    callbackasm1(SB)
	MOVD $1658, R12
	B    callbackasm1(SB)
	MOVD $1659, R12
	B    callbackasm1(SB)
	MOVD $1660, R12
	B    callbackasm1(SB)
	MOVD $1661, R12
	B    callbackasm1(SB)
	MOVD $1662, R12
	B    callbackasm1(SB)
	MOVD $1663, R12
	B    callbackasm1(SB)
	MOVD $1664, R12
	B    callbackasm1(SB)
	MOVD $1665, R12
	B    callbackasm1(SB)
	MOVD $1666, R12
	B    callbackasm1(SB)
	MOVD $1667, R12
	B    callbackasm1(SB)
	MOVD $1668, R12
	B    callbackasm1(SB)
	MOVD $1669, R12
	B    callbackasm1(SB)
	MOVD $1670, R12
	B    callbackasm1(SB)
	MOVD $1671, R12
	B    callbackasm1(SB)
	MOVD $1672, R12
	B    callbackasm1(SB)
	MOVD $1673, R12
	B    callbackasm1(SB)
	MOVD $1674, R12
	B    callbackasm1(SB)
	MOVD $1675, R12
	B    callbackasm1(SB)
	MOVD $1676, R12
	B    callbackasm1(SB)
	MOVD $1677, R12
	B    callbackasm1(SB)
	MOVD $1678, R12
	B    callbackasm1(SB)
	MOVD $1679, R12
	B    callbackasm1(SB)
	MOVD $1680, R12
	B    callbackasm1(SB)
	MOVD $1681, R12
	B    callbackasm1(SB)
	MOVD $1682, R12
	B    callbackasm1(SB)
	MOVD $1683, R12
	B    callbackasm1(SB)
	MOVD $1684, R12
	B    callbackasm1(SB)
	MOVD $1685, R12
	B    callbackasm1(SB)
	MOVD $1686, R12
	B    callbackasm1(SB)
	MOVD $1687, R12
	B    callbackasm1(SB)
	MOVD $1688, R12
	B    callbackasm1(SB)
	MOVD $1689, R12
	B    callbackasm1(SB)
	MOVD $1690, R12
	B    callbackasm1(SB)
	MOVD $1691, R12
	B    callbackasm1(SB)
	MOVD $1692, R12
	B    callbackasm1(SB)
	MOVD $1693, R12
	B    callbackasm1(SB)
	MOVD $1694, R12
	B    callbackasm1(SB)
	MOVD $1695, R12
	B    callbackasm1(SB)
	MOVD $1696, R12
	B    callbackasm1(SB)
	MOVD $1697, R12
	B    callbackasm1(SB)
	MOVD $1698, R12
	B    callbackasm1(SB)
	MOVD $1699, R12
	B    callbackasm1(SB)
	MOVD $1700, R12
	B    callbackasm1(SB)
	MOVD $1701, R12
	B    callbackasm1(SB)
	MOVD $1702, R12
	B    callbackasm1(SB)
	MOVD $1703, R12
	B    callbackasm1(SB)
	MOVD $1704, R12
	B    callbackasm1(SB)
	MOVD $1705, R12
	B    callbackasm1(SB)
	MOVD $1706, R12
	B    callbackasm1(SB)
	MOVD $1707, R12
	B    callbackasm1(SB)
	MOVD $1708, R12
	B    callbackasm1(SB)
	MOVD $1709, R12
	B    callbackasm1(SB)
	MOVD $1710, R12
	B    callbackasm1(SB)
	MOVD $1711, R12
	B    callbackasm1(SB)
	MOVD $1712, R12
	B    callbackasm1(SB)
	MOVD $1713, R12
	B    callbackasm1(SB)
	MOVD $1714, R12
	B    callbackasm1(SB)
	MOVD $1715, R12
	B    callbackasm1(SB)
	MOVD $1716, R12
	B    callbackasm1(SB)
	MOVD $1717, R12
	B    callbackasm1(SB)
	MOVD $1718, R12
	B    callbackasm1(SB)
	MOVD $1719, R12
	B    callbackasm1(SB)
	MOVD $1720, R12
	B    callbackasm1(SB)
	MOVD $1721, R12
	B    callbackasm1(SB)
	MOVD $1722, R12
	B    callbackasm1(SB)
	MOVD $1723, R12
	B    callbackasm1(SB)
	MOVD $1724, R12
	B    callbackasm1(SB)
	MOVD $1725, R12
	B    callbackasm1(SB)
	MOVD $1726, R12
	B    callbackasm1(SB)
	MOVD $1727, R12
	B    callbackasm1(SB)
	MOVD $1728, R12
	B    callbackasm1(SB)
	MOVD $1729, R12
	B    callbackasm1(SB)
	MOVD $1730, R12
	B    callbackasm1(SB)
	MOVD $1731, R12
	B    callbackasm1(SB)
	MOVD $1732, R12
	B    callbackasm1(SB)
	MOVD $1733, R12
	B    callbackasm1(SB)
	MOVD $1734, R12
	B    callbackasm1(SB)
	MOVD $1735, R12
	B    callbackasm1(SB)
	MOVD $1736, R12
	B    callbackasm1(SB)
	MOVD $1737, R12
	B    callbackasm1(SB)
	MOVD $1738, R12
	B    callbackasm1(SB)
	MOVD $1739, R12
	B    callbackasm1(SB)
	MOVD $1740, R12
	B    callbackasm1(SB)
	MOVD $1741, R12
	B    callbackasm1(SB)
	MOVD $1742, R12
	B    callbackasm1(SB)
	MOVD $1743, R12
	B    callbackasm1(SB)
	MOVD $1744, R12
	B    callbackasm1(SB)
	MOVD $1745, R12
	B    callbackasm1(SB)
	MOVD $1746, R12
	B    callbackasm1(SB)
	MOVD $1747, R12
	B    callbackasm1(SB)
	MOVD $1748, R12
	B    callbackasm1(SB)
	MOVD $1749, R12
	B    callbackasm1(SB)
	MOVD $1750, R12
	B    callbackasm1(SB)
	MOVD $1751, R12
	B    callbackasm1(SB)
	MOVD $1752, R12
	B    callbackasm1(SB)
	MOVD $1753, R12
	B    callbackasm1(SB)
	MOVD $1754, R12
	B    callbackasm1(SB)
	MOVD $1755, R12
	B    callbackasm1(SB)
	MOVD $1756, R12
	B    callbackasm1(SB)
	MOVD $1757, R12
	B    callbackasm1(SB)
	MOVD $1758, R12
	B    callbackasm1(SB)
	MOVD $1759, R12
	B    callbackasm1(SB)
	MOVD $1760, R12
	B    callbackasm1(SB)
	MOVD $1761, R12
	B    callbackasm1(SB)
	MOVD $1762, R12
	B    callbackasm1(SB)
	MOVD $1763, R12
	B    callbackasm1(SB)
	MOVD $1764, R12
	B    callbackasm1(SB)
	MOVD $1765, R12
	B    callbackasm1(SB)
	MOVD $1766, R12
	B    callbackasm1(SB)
	MOVD $1767, R12
	B    callbackasm1(SB)
	MOVD $1768, R12
	B    callbackasm1(SB)
	MOVD $1769, R12
	B    callbackasm1(SB)
	MOVD $1770, R12
	B    callbackasm1(SB)
	MOVD $1771, R12
	B    callbackasm1(SB)
	MOVD $1772, R12
	B    callbackasm1(SB)
	MOVD $1773, R12
	B    callbackasm1(SB)
	MOVD $1774, R12
	B    callbackasm1(SB)
	MOVD $1775, R12
	B    callbackasm1(SB)
	MOVD $1776, R12
	B    callbackasm1(SB)
	MOVD $1777, R12
	B    callbackasm1(SB)
	MOVD $1778, R12
	B    callbackasm1(SB)
	MOVD $1779, R12
	B    callbackasm1(SB)
	MOVD $1780, R12
	B    callbackasm1(SB)
	MOVD $1781, R12
	B    callbackasm1(SB)
	MOVD $1782, R12
	B    callbackasm1(SB)
	MOVD $1783, R12
	B    callbackasm1(SB)
	MOVD $1784, R12
	B    callbackasm1(SB)
	MOVD $1785, R12
	B    callbackasm1(SB)
	MOVD $1786, R12
	B    callbackasm1(SB)
	MOVD $1787, R12
	B    callbackasm1(SB)
	MOVD $1788, R12
	B    callbackasm1(SB)
	MOVD $1789, R12
	B    callbackasm1(SB)
	MOVD $1790, R12
	B    callbackasm1(SB)
	MOVD $1791, R12
	B    callbackasm1(SB)
	MOVD $1792, R12
	B    callbackasm1(SB)
	MOVD $1793, R12
	B    callbackasm1(SB)
	MOVD $1794, R12
	B    callbackasm1(SB)
	MOVD $1795, R12
	B    callbackasm1(SB)
	MOVD $1796, R12
	B    callbackasm1(SB)
	MOVD $1797, R12
	B    callbackasm1(SB)
	MOVD $1798, R12
	B    callbackasm1(SB)
	MOVD $1799, R12
	B    callbackasm1(SB)
	MOVD $1800, R12
	B    callbackasm1(SB)
	MOVD $1801, R12
	B    callbackasm1(SB)
	MOVD $1802, R12
	B    callbackasm1(SB)
	MOVD $1803, R12
	B    callbackasm1(SB)
	MOVD $1804, R12
	B    callbackasm1(SB)
	MOVD $1805, R12
	B    callbackasm1(SB)
	MOVD $1806, R12
	B    callbackasm1(SB)
	MOVD $1807, R12
	B    callbackasm1(SB)
	MOVD $1808, R12
	B    callbackasm1(SB)
	MOVD $1809, R12
	B    callbackasm1(SB)
	MOVD $1810, R12
	B    callbackasm1(SB)
	MOVD $1811, R12
	B    callbackasm1(SB)
	MOVD $1812, R12
	B    callbackasm1(SB)
	MOVD $1813, R12
	B    callbackasm1(SB)
	MOVD $1814, R12
	B    callbackasm1(SB)
	MOVD $1815, R12
	B    callbackasm1(SB)
	MOVD $1816, R12
	B    callbackasm1(SB)
	MOVD $1817, R12
	B    callbackasm1(SB)
	MOVD $1818, R12
	B    callbackasm1(SB)
	MOVD $1819, R12
	B    callbackasm1(SB)
	MOVD $1820, R12
	B    callbackasm1(SB)
	MOVD $1821, R12
	B    callbackasm1(SB)
	MOVD $1822, R12
	B    callbackasm1(SB)
	MOVD $1823, R12
	B    callbackasm1(SB)
	MOVD $1824, R12
	B    callbackasm1(SB)
	MOVD $1825, R12
	B    callbackasm1(SB)
	MOVD $1826, R12
	B    callbackasm1(SB)
	MOVD $1827, R12
	B    callbackasm1(SB)
	MOVD $1828, R12
	B    callbackasm1(SB)
	MOVD $1829, R12
	B    callbackasm1(SB)
	MOVD $1830, R12
	B    callbackasm1(SB)
	MOVD $1831, R12
	B    callbackasm1(SB)
	MOVD $1832, R12
	B    callbackasm1(SB)
	MOVD $1833, R12
	B    callbackasm1(SB)
	MOVD $1834, R12
	B    callbackasm1(SB)
	MOVD $1835, R12
	B    callbackasm1(SB)
	MOVD $1836, R12
	B    callbackasm1(SB)
	MOVD $1837, R12
	B    callbackasm1(SB)
	MOVD $1838, R12
	B    callbackasm1(SB)
	MOVD $1839, R12
	B    callbackasm1(SB)
	MOVD $1840, R12
	B    callbackasm1(SB)
	MOVD $1841, R12
	B    callbackasm1(SB)
	MOVD $1842, R12
	B    callbackasm1(SB)
	MOVD $1843, R12
	B    callbackasm1(SB)
	MOVD $1844, R12
	B    callbackasm1(SB)
	MOVD $1845, R12
	B    callbackasm1(SB)
	MOVD $1846, R12
	B    callbackasm1(SB)
	MOVD $1847, R12
	B    callbackasm1(SB)
	MOVD $1848, R12
	B    callbackasm1(SB)
	MOVD $1849, R12
	B    callbackasm1(SB)
	MOVD $1850, R12
	B    callbackasm1(SB)
	MOVD $1851, R12
	B    callbackasm1(SB)
	MOVD $1852, R12
	B    callbackasm1(SB)
	MOVD $1853, R12
	B    callbackasm1(SB)
	MOVD $1854, R12
	B    callbackasm1(SB)
	MOVD $1855, R12
	B    callbackasm1(SB)
	MOVD $1856, R12
	B    callbackasm1(SB)
	MOVD $1857, R12
	B    callbackasm1(SB)
	MOVD $1858, R12
	B    callbackasm1(SB)
	MOVD $1859, R12
	B    callbackasm1(SB)
	MOVD $1860, R12
	B    callbackasm1(SB)
	MOVD $1861, R12
	B    callbackasm1(SB)
	MOVD $1862, R12
	B    callbackasm1(SB)
	MOVD $1863, R12
	B    callbackasm1(SB)
	MOVD $1864, R12
	B    callbackasm1(SB)
	MOVD $1865, R12
	B    callbackasm1(SB)
	MOVD $1866, R12
	B    callbackasm1(SB)
	MOVD $1867, R12
	B    callbackasm1(SB)
	MOVD $1868, R12
	B    callbackasm1(SB)
	MOVD $1869, R12
	B    callbackasm1(SB)
	MOVD $1870, R12
	B    callbackasm1(SB)
	MOVD $1871, R12
	B    callbackasm1(SB)
	MOVD $1872, R12
	B    callbackasm1(SB)
	MOVD $1873, R12
	B    callbackasm1(SB)
	MOVD $1874, R12
	B    callbackasm1(SB)
	MOVD $1875, R12
	B    callbackasm1(SB)
	MOVD $1876, R12
	B    callbackasm1(SB)
	MOVD $1877, R12
	B    callbackasm1(SB)
	MOVD $1878, R12
	B    callbackasm1(SB)
	MOVD $1879, R12
	B    callbackasm1(SB)
	MOVD $1880, R12
	B    callbackasm1(SB)
	MOVD $1881, R12
	B    callbackasm1(SB)
	MOVD $1882, R12
	B    callbackasm1(SB)
	MOVD $1883, R12
	B    callbackasm1(SB)
	MOVD $1884, R12
	B    callbackasm1(SB)
	MOVD $1885, R12
	B    callbackasm1(SB)
	MOVD $1886, R12
	B    callbackasm1(SB)
	MOVD $1887, R12
	B    callbackasm1(SB)
	MOVD $1888, R12
	B    callbackasm1(SB)
	MOVD $1889, R12
	B    callbackasm1(SB)
	MOVD $1890, R12
	B    callbackasm1(SB)
	MOVD $1891, R12
	B    callbackasm1(SB)
	MOVD $1892, R12
	B    callbackasm1(SB)
	MOVD $1893, R12
	B    callbackasm1(SB)
	MOVD $1894, R12
	B    callbackasm1(SB)
	MOVD $1895, R12
	B    callbackasm1(SB)
	MOVD $1896, R12
	B    callbackasm1(SB)
	MOVD $1897, R12
	B    callbackasm1(SB)
	MOVD $1898, R12
	B    callbackasm1(SB)
	MOVD $1899, R12
	B    callbackasm1(SB)
	MOVD $1900, R12
	B    callbackasm1(SB)
	MOVD $1901, R12
	B    callbackasm1(SB)
	MOVD $1902, R12
	B    callbackasm1(SB)
	MOVD $1903, R12
	B    callbackasm1(SB)
	MOVD $1904, R12
	B    callbackasm1(SB)
	MOVD $1905, R12
	B    callbackasm1(SB)
	MOVD $1906, R12
	B    callbackasm1(SB)
	MOVD $1907, R12
	B    callbackasm1(SB)
	MOVD $1908, R12
	B    callbackasm1(SB)
	MOVD $1909, R12
	B    callbackasm1(SB)
	MOVD $1910, R12
	B    callbackasm1(SB)
	MOVD $1911, R12
	B    callbackasm1(SB)
	MOVD $1912, R12
	B    callbackasm1(SB)
	MOVD $1913, R12
	B    callbackasm1(SB)
	MOVD $1914, R12
	B    callbackasm1(SB)
	MOVD $1915, R12
	B    callbackasm1(SB)
	MOVD $1916, R12
	B    callbackasm1(SB)
	MOVD $1917, R12
	B    callbackasm1(SB)
	MOVD $1918, R12
	B    callbackasm1(SB)
	MOVD $1919, R12
	B    callbackasm1(SB)
	MOVD $1920, R12
	B    callbackasm1(SB)
	MOVD $1921, R12
	B    callbackasm1(SB)
	MOVD $1922, R12
	B    callbackasm1(SB)
	MOVD $1923, R12
	B    callbackasm1(SB)
	MOVD $1924, R12
	B    callbackasm1(SB)
	MOVD $1925, R12
	B    callbackasm1(SB)
	MOVD $1926, R12
	B    callbackasm1(SB)
	MOVD $1927, R12
	B    callbackasm1(SB)
	MOVD $1928, R12
	B    callbackasm1(SB)
	MOVD $1929, R12
	B    callbackasm1(SB)
	MOVD $1930, R12
	B    callbackasm1(SB)
	MOVD $1931, R12
	B    callbackasm1(SB)
	MOVD $1932, R12
	B    callbackasm1(SB)
	MOVD $1933, R12
	B    callbackasm1(SB)
	MOVD $1934, R12
	B    callbackasm1(SB)
	MOVD $1935, R12
	B    callbackasm1(SB)
	MOVD $1936, R12
	B    callbackasm1(SB)
	MOVD $1937, R12
	B    callbackasm1(SB)
	MOVD $1938, R12
	B    callbackasm1(SB)
	MOVD $1939, R12
	B    callbackasm1(SB)
	MOVD $1940, R12
	B    callbackasm1(SB)
	MOVD $1941, R12
	B    callbackasm1(SB)
	MOVD $1942, R12
	B    callbackasm1(SB)
	MOVD $1943, R12
	B    callbackasm1(SB)
	MOVD $1944, R12
	B    callbackasm1(SB)
	MOVD $1945, R12
	B    callbackasm1(SB)
	MOVD $1946, R12
	B    callbackasm1(SB)
	MOVD $1947, R12
	B    callbackasm1(SB)
	MOVD $1948, R12
	B    callbackasm1(SB)
	MOVD $1949, R12
	B    callbackasm1(SB)
	MOVD $1950, R12
	B    callbackasm1(SB)
	MOVD $1951, R12
	B    callbackasm1(SB)
	MOVD $1952, R12
	B    callbackasm1(SB)
	MOVD $1953, R12
	B    callbackasm1(SB)
	MOVD $1954, R12
	B    callbackasm1(SB)
	MOVD $1955, R12
	B    callbackasm1(SB)
	MOVD $1956, R12
	B    callbackasm1(SB)
	MOVD $1957, R12
	B    callbackasm1(SB)
	MOVD $1958, R12
	B    callbackasm1(SB)
	MOVD $1959, R12
	B    callbackasm1(SB)
	MOVD $1960, R12
	B    callbackasm1(SB)
	MOVD $1961, R12
	B    callbackasm1(SB)
	MOVD $1962, R12
	B    callbackasm1(SB)
	MOVD $1963, R12
	B    callbackasm1(SB)
	MOVD $1964, R12
	B    callbackasm1(SB)
	MOVD $1965, R12
	B    callbackasm1(SB)
	MOVD $1966, R12
	B    callbackasm1(SB)
	MOVD $1967, R12
	B    callbackasm1(SB)
	MOVD $1968, R12
	B    callbackasm1(SB)
	MOVD $1969, R12
	B    callbackasm1(SB)
	MOVD $1970, R12
	B    callbackasm1(SB)
	MOVD $1971, R12
	B    callbackasm1(SB)
	MOVD $1972, R12
	B    callbackasm1(SB)
	MOVD $1973, R12
	B    callbackasm1(SB)
	MOVD $1974, R12
	B    callbackasm1(SB)
	MOVD $1975, R12
	B    callbackasm1(SB)
	MOVD $1976, R12
	B    callbackasm1(SB)
	MOVD $1977, R12
	B    callbackasm1(SB)
	MOVD $1978, R12
	B    callbackasm1(SB)
	MOVD $1979, R12
	B    callbackasm1(SB)
	MOVD $1980, R12
	B    callbackasm1(SB)
	MOVD $1981, R12
	B    callbackasm1(SB)
	MOVD $1982, R12
	B    callbackasm1(SB)
	MOVD $1983, R12
	B    callbackasm1(SB)
	MOVD $1984, R12
	B    callbackasm1(SB)
	MOVD $1985, R12
	B    callbackasm1(SB)
	MOVD $1986, R12
	B    callbackasm1(SB)
	MOVD $1987, R12
	B    callbackasm1(SB)
	MOVD $1988, R12
	B    callbackasm1(SB)
	MOVD $1989, R12
	B    callbackasm1(SB)
	MOVD $1990, R12
	B    callbackasm1(SB)
	MOVD $1991, R12
	B    callbackasm1(SB)
	MOVD $1992, R12
	B    callbackasm1(SB)
	MOVD $1993, R12
	B    callbackasm1(SB)
	MOVD $1994, R12
	B    callbackasm1(SB)
	MOVD $1995, R12
	B    callbackasm1(SB)
	MOVD $1996, R12
	B    callbackasm1(SB)
	MOVD $1997, R12
	B    callbackasm1(SB)
	MOVD $1998, R12
	B    callbackasm1(SB)
	MOVD $1999, R12
	B    callbackasm1(SB)
