# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Literal, Optional

from pydantic import Field

from wandb._pydantic import GQLBase, Typename

from .enums import ArtifactCollectionState


class DeleteArtifactPortfolio(GQLBase):
    delete_artifact_portfolio: Optional[
        DeleteArtifactPortfolioDeleteArtifactPortfolio
    ] = Field(alias="deleteArtifactPortfolio")


class DeleteArtifactPortfolioDeleteArtifactPortfolio(GQLBase):
    artifact_collection: DeleteArtifactPortfolioDeleteArtifactPortfolioArtifactCollection = Field(
        alias="artifactCollection"
    )


class DeleteArtifactPortfolioDeleteArtifactPortfolioArtifactCollection(GQLBase):
    typename__: Typename[
        Literal["ArtifactCollection", "ArtifactPortfolio", "ArtifactSequence"]
    ]
    state: ArtifactCollectionState


DeleteArtifactPortfolio.model_rebuild()
DeleteArtifactPortfolioDeleteArtifactPortfolio.model_rebuild()
