# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Optional

from pydantic import Field

from wandb._pydantic import GQLBase

from .fragments import ArtifactCollectionsFragment


class ProjectArtifactCollections(GQLBase):
    project: Optional[ProjectArtifactCollectionsProject]


class ProjectArtifactCollectionsProject(GQLBase):
    artifact_type: Optional[ProjectArtifactCollectionsProjectArtifactType] = Field(
        alias="artifactType"
    )


class ProjectArtifactCollectionsProjectArtifactType(GQLBase):
    artifact_collections: Optional[ArtifactCollectionsFragment] = Field(
        alias="artifactCollections"
    )


ProjectArtifactCollections.model_rebuild()
ProjectArtifactCollectionsProject.model_rebuild()
ProjectArtifactCollectionsProjectArtifactType.model_rebuild()
