# Generated by ariadne-codegen
# Source: tools/graphql_codegen/automations/

from __future__ import annotations

from typing import Optional

from pydantic import Field

from wandb._pydantic import GQLBase

from .fragments import ProjectConnectionFields


class GetAutomationsByEntity(GQLBase):
    search_scope: Optional[GetAutomationsByEntitySearchScope] = Field(
        alias="searchScope"
    )


class GetAutomationsByEntitySearchScope(GQLBase):
    projects: Optional[ProjectConnectionFields]


GetAutomationsByEntity.model_rebuild()
GetAutomationsByEntitySearchScope.model_rebuild()
