# Generated by ariadne-codegen
# Source: tools/graphql_codegen/automations/

from __future__ import annotations

from typing import Optional

from wandb._pydantic import GQLBase

from .fragments import GenericWebhookIntegrationConnectionFields


class GenericWebhookIntegrationsByEntity(GQLBase):
    entity: Optional[GenericWebhookIntegrationsByEntityEntity]


class GenericWebhookIntegrationsByEntityEntity(GQLBase):
    integrations: Optional[GenericWebhookIntegrationConnectionFields]


GenericWebhookIntegrationsByEntity.model_rebuild()
GenericWebhookIntegrationsByEntityEntity.model_rebuild()
