# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import List, Optional

from pydantic import Field

from wandb._pydantic import GQLBase, GQLId


class CreateArtifactCollectionTagAssignments(GQLBase):
    create_artifact_collection_tag_assignments: Optional[
        CreateArtifactCollectionTagAssignmentsCreateArtifactCollectionTagAssignments
    ] = Field(alias="createArtifactCollectionTagAssignments")


class CreateArtifactCollectionTagAssignmentsCreateArtifactCollectionTagAssignments(
    GQLBase
):
    tags: List[
        CreateArtifactCollectionTagAssignmentsCreateArtifactCollectionTagAssignmentsTags
    ]


class CreateArtifactCollectionTagAssignmentsCreateArtifactCollectionTagAssignmentsTags(
    GQLBase
):
    id: GQLId
    name: str
    tag_category_name: str = Field(alias="tagCategoryName")


CreateArtifactCollectionTagAssignments.model_rebuild()
CreateArtifactCollectionTagAssignmentsCreateArtifactCollectionTagAssignments.model_rebuild()
