# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Optional

from pydantic import Field

from wandb._pydantic import GQLBase


class DeleteAliases(GQLBase):
    delete_aliases: Optional[DeleteAliasesDeleteAliases] = Field(alias="deleteAliases")


class DeleteAliasesDeleteAliases(GQLBase):
    success: bool


DeleteAliases.model_rebuild()
