# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Literal, Optional

from pydantic import Field

from wandb._pydantic import GQLBase, GQLId, Typename


class UpdateArtifactSequence(GQLBase):
    update_artifact_sequence: Optional[UpdateArtifactSequenceUpdateArtifactSequence] = (
        Field(alias="updateArtifactSequence")
    )


class UpdateArtifactSequenceUpdateArtifactSequence(GQLBase):
    artifact_collection: UpdateArtifactSequenceUpdateArtifactSequenceArtifactCollection = Field(
        alias="artifactCollection"
    )


class UpdateArtifactSequenceUpdateArtifactSequenceArtifactCollection(GQLBase):
    typename__: Typename[
        Literal["ArtifactCollection", "ArtifactPortfolio", "ArtifactSequence"]
    ]
    id: GQLId
    name: str
    description: Optional[str]


UpdateArtifactSequence.model_rebuild()
UpdateArtifactSequenceUpdateArtifactSequence.model_rebuild()
