# Generated by ariadne-codegen
# Source: tools/graphql_codegen/automations/

from __future__ import annotations

from typing import Optional

from wandb._pydantic import GQLBase

from .fragments import SlackIntegrationConnectionFields


class SlackIntegrationsByEntity(GQLBase):
    entity: Optional[SlackIntegrationsByEntityEntity]


class SlackIntegrationsByEntityEntity(GQLBase):
    integrations: Optional[SlackIntegrationConnectionFields]


SlackIntegrationsByEntity.model_rebuild()
SlackIntegrationsByEntityEntity.model_rebuild()
