# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Optional

from pydantic import Field

from wandb._pydantic import GQLBase

from .fragments import ArtifactFragment


class UpdateArtifact(GQLBase):
    update_artifact: Optional[UpdateArtifactUpdateArtifact] = Field(
        alias="updateArtifact"
    )


class UpdateArtifactUpdateArtifact(GQLBase):
    artifact: ArtifactFragment


UpdateArtifact.model_rebuild()
UpdateArtifactUpdateArtifact.model_rebuild()
