# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Optional

from pydantic import Field

from wandb._pydantic import GQLBase

from .fragments import FilesFragment


class ArtifactVersionFiles(GQLBase):
    project: Optional[ArtifactVersionFilesProject]


class ArtifactVersionFilesProject(GQLBase):
    artifact_type: Optional[ArtifactVersionFilesProjectArtifactType] = Field(
        alias="artifactType"
    )


class ArtifactVersionFilesProjectArtifactType(GQLBase):
    artifact: Optional[ArtifactVersionFilesProjectArtifactTypeArtifact]


class ArtifactVersionFilesProjectArtifactTypeArtifact(GQLBase):
    files: Optional[FilesFragment]


ArtifactVersionFiles.model_rebuild()
ArtifactVersionFilesProject.model_rebuild()
ArtifactVersionFilesProjectArtifactType.model_rebuild()
ArtifactVersionFilesProjectArtifactTypeArtifact.model_rebuild()
