# Generated by ariadne-codegen
# Source: tools/graphql_codegen/automations/

from __future__ import annotations

from typing import Optional

from wandb._pydantic import GQLBase

from .fragments import CreateAutomationResult


class CreateAutomation(GQLBase):
    result: Optional[CreateAutomationResult]


CreateAutomation.model_rebuild()
