query FetchOrgEntityFromEntity($entityName: String!) {
    entity(name: $entityName) {
        organization {
            name
            orgEntity {
                name
            }
        }
    }
}
